{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwColors.pas
 Description: System and own color functions
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     14.12.2002
 Last mod.:   05.03.2003
 History:     n/a
-----------------------------------------------------------------------------}

unit dwColors;

interface

uses dwTypes, Graphics;

function dwActiveCaptionGradientColor: TColor;
function dwHotLightColor: TColor;
function dwInactiveCaptionGradientColor: TColor;
function dwInverseColor(const Color: TColor): TColor;
function dwManipulateColor(const Color: TColor; const Red, Green, Blue: Byte): TColor;
function dwMenuBarColor: TColor;
function dwMenuHilightColor: TColor;
function dwUseLightOrDark(const Color: TColor): TDWUseLightDarkColor;


implementation

uses dwWinSystem,
     Windows;

function dwActiveCaptionGradientColor: TColor;
const
  COLOR_GRADIENTACTIVECAPTION = 27;
begin
   result:= clNone;
   if dwIsWin95 or dwIsNT351 or dwIsNT4 then exit;
   result:= GetSysColor(COLOR_GRADIENTACTIVECAPTION);
end;

function dwHotLightColor: TColor;
const
  COLOR_HOTLIGHT = 26;
begin
   result:= clNone;
   if dwIsWin95 or dwIsNT351 or dwIsNT4 then exit;
   result:= GetSysColor(COLOR_HOTLIGHT);
end;

function dwInactiveCaptionGradientColor: TColor;
const
  COLOR_GRADIENTINACTIVECAPTION = 28;
begin
   result:= clNone;
   if dwIsWin95 or dwIsNT351 or dwIsNT4 then exit;
   result:= GetSysColor(COLOR_GRADIENTINACTIVECAPTION);
end;

function dwInverseColor(const Color: TColor): TColor;
begin
  if Color < 0 then
    Result := GetSysColor(Color and $ff) xor $ffffff
  else
    Result := Color xor $ffffff;
end;

function dwManipulateColor(const Color: TColor; const Red, Green, Blue: Byte): TColor;
type
  TColorBlock = record
    case Integer of
      0: (Value: Longint);
      1: (Red, Green, Blue: Byte);
    end;
var
	iR, iG, iB: Byte;
   Col: TColorBlock;
begin
   Col.Value:= Color;
   Col.Red:= Red;
   Col.Green:= Green;
   Col.Blue:= Blue;
   result:= Col.Value;
end;

function dwMenuBarColor: TColor;
const
  COLOR_MENUBAR = 30;
begin
   result:= clNone;
   if not dwIsWinXP then exit;
   result:= GetSysColor(COLOR_MENUBAR);
end;

function dwMenuHilightColor: TColor;
const
  COLOR_MENUHILIGHT = 29;
begin
   result:= clNone;
   if not dwIsWinXP then exit;
   result:= GetSysColor(COLOR_MENUHILIGHT);
end;

function dwUseLightOrDark(const Color: TColor): TDWUseLightDarkColor;
var
   R, G, B: Integer;
begin
   R := GetRValue(Color) * 2;
   G := GetGValue(Color) * 5;
   B := GetBValue(Color);
   if R + G + B < 1024 then
      result:= dwuldcLight
   else
      result:= dwuldcDark;
end;

end.
