{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwDateTime.pas
 Description: Date and time helpers
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     15.12.2002
 Last mod.:   16.08.2004
 History:     n/a
-----------------------------------------------------------------------------}

unit dwDateTime;

interface

uses dwTypes;

function dwDayOfYear (ADate : TDateTime): Integer;
function dwGetHoliday(const Year: Word; const Holiday: TDWHoliday): TDateTime;
function dwTimeIsAM(ProofTime: TDateTime): Boolean;
function dwWeekOfYear(ProofDate: TDateTime): Byte;
function dwYearToThai(const Year: Word): Word;

procedure dwSetSystemClock(Year, Month, Day, Hour, Minute, Second, MSecond: Integer);

implementation

uses dwInternal,
     SysUtils, Windows;

procedure dwSetSystemClock(Year, Month, Day, Hour, Minute, Second, MSecond: Integer);
var
   SysTime: TSystemTime;
   DayTime: TDateTime;
begin
   DayTime:= EncodeDate(Year, Month, Day) +
             EncodeTime(Hour, Minute, Second, MSecond);
             DateTimeToSystemTime(DayTime, SysTime);
             SetLocalTime(SysTime);
end;

function dwWeekOfYear(ProofDate: TDateTime): Byte;
const
   FirstWeekDay = 2;
   FirstWeekDate = 4;
var
   Year, Month, Day: Word;
begin
   ProofDate:= ProofDate - ((DayOfWeek(ProofDate) - FirstWeekDay + 7) mod 7) + 7 - FirstWeekDate;
   DecodeDate(ProofDate, Year, Month, Day);
   result:= (Trunc(ProofDate - EncodeDate(Year, 1, 1)) div 7) + 1;
end;

function dwGetHoliday(const Year: Word; const Holiday: TDWHoliday): TDateTime;
var
   BDate, EDate, WDate: TDateTime;
begin
   result:= 0;
   BDate:= EncodeDate(Year, 11, 1) + ((11 - DayOfWeek(EncodeDate(Year, 11, 1))) mod 7) + 14;
   EDate:= dwiGetEasterDate(Year);
   WDate:= EncodeDate(Year, 12, 24);
   case Holiday of
   dwhdNeujahr:                      result:= EncodeDate(Year,  1,  1);
   dwhdValentinstag:                 result:= EncodeDate(Year,  2, 14);
   dwhdTagDerArbeit:                 result:= EncodeDate(Year,  5,  1);
   dwhdTagDerDeutschenEinheit:       result:= EncodeDate(Year, 10,  3);
   dwhdReformationstag:              result:= EncodeDate(Year, 31, 10);
   dwhdHeiligabend:                  result:= EncodeDate(Year, 12, 24);
   dwhdWeihnachtsfeiertag1:          result:= EncodeDate(Year, 12, 25);
   dwhdWeihnachtsfeiertag2:          result:= EncodeDate(Year, 12, 26);
   dwhdSilvester:                    result:= EncodeDate(Year, 12, 31);
   dwhdKarfreitag:                   result:= EDate - 2;
   dwhdOsterSonntag:                 result:= EDate;
   dwhdOstermontag:                  result:= EDate + 1;
   dwhdHimmelfahrt:                  result:= EDate + 39;
   dwhdPfingstsonntag:               result:= EDate + 49;
   dwhdPfingstmontag:                result:= EDate + 50;
   dwhdFronleichnam:                 result:= EDate + 60;
   dwhdErntedankfest:                result:= BDate - 45;
   dwhdBussUndBetTag:                result:= BDate;
   dwhdAdvent1:                      result:= dwiSundayBeforeDate(WDate) - 21;
   dwhdAdvent2:                      result:= dwiSundayBeforeDate(WDate) - 14;
   dwhdAdvent3:                      result:= dwiSundayBeforeDate(WDate) - 7;
   dwhdAdvent4:                      result:= dwiSundayBeforeDate(WDate);
   end;
end;

function dwYearToThai(const Year: Word): Word;
begin
   result:= Year + 543;
end;

function dwTimeIsAM(ProofTime: TDateTime): Boolean;
begin
   result:= (Frac(Time)=0);
end;

function dwDayOfYear (ADate : TDateTime): Integer;
var
   y,m,d : Word;
begin
   DecodeDate(ADate,y,m,d);
   result := Trunc(ADate) - Trunc(EncodeDate (y, 1, 1)) + 1;
end;

end.
 