{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwDialogs.pas
 Description: System and own dialogs
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     14.12.2002
 Last mod.:   16.08.2004
 History:     n/a
-----------------------------------------------------------------------------}
unit dwDialogs;

interface

uses dwTypes,
     Windows;

function dwShowComputerBrowserDialog(ParentWindow: HWND; const DisplayText: String; const NewStyle: Boolean): String;
function dwShowFolderBrowserDialog(ParentWindow: HWND; const DisplayText: String; const Editable, NewStyle, ShowFiles: Boolean): String; overload;
function dwShowFolderBrowserDialog(ParentWindow: HWND; const DisplayText, InitPath: String; const Editable, NewStyle, ShowFiles: Boolean): String; overload;

procedure dwFileOpenWithDialog(const Filename: String);
procedure dwFilePropertiesDialog(const Filename: String);
procedure dwShowFormatDriveDialog(const Drive: Char; const Capacity: TDWFmtDriveDlgCapacity; const QuickErase: Boolean);
procedure dwShowInteger(Int: Integer);
procedure dwShowPoint(Point: TPoint);
procedure dwShowRect(Rect: TRect);
procedure dwShowSpecialDialog(const SpecialDialog: TDWWindowsDialog);
procedure dwStartNewBrowserWindow(const URL: string);

implementation

uses dwConsts, dwFiles, dwInternal, dwWinSystem,
     DDEMan, Dialogs, Messages, ShellAPI, SysUtils, ShlObj;

function dwShowComputerBrowserDialog(ParentWindow: HWND; const DisplayText: String; const NewStyle: Boolean): String;
var
	BI: TBrowseInfo;
   Flags: LongWord;
   SelectedPIDL, PrinterPIDL: PItemIDList;
   DName: array[0..MAX_PATH] of Char;
//   Path: array[0..MAX_PATH] of Char;

   procedure MergeFlag(var Flag: LongWord; const Value: LongWord);
   begin
   	Flag:= Flag or Value;
   end;
begin
   Flags:= 0;
   SHGetSpecialFolderLocation(ParentWindow, $12, PrinterPIDL);
   ZeroMemory(@BI, SizeOf(TBrowseInfo));
   MergeFlag(Flags, BIF_BROWSEFORCOMPUTER);
   if NewStyle then MergeFlag(Flags, BIF_NEWDIALOGSTYLE);
   MergeFlag(Flags, BIF_NONEWFOLDERBUTTON);
   with BI do begin
      hwndOwner:= ParentWindow;
      pidlRoot:= PrinterPIDL;
      pszDisplayName:= @DName;
      lpszTitle:= PChar(DisplayText);
      ulFlags:= Flags;
      lpfn:= NIL;
      lParam:= 0;
   end;
   SelectedPIDL:= SHBrowseForFolder(bi);
   if SelectedPIDL <> NIL then begin
      	result:= StrPas(BI.pszDisplayName);
   end;
//   FreeMem(BI.pszDisplayName);
   GlobalFreePtr(SelectedPIDL);
   GlobalFreePtr(PrinterPIDL);
end;

function dwShowFolderBrowserDialog(ParentWindow: HWND; const DisplayText: String; const Editable, NewStyle, ShowFiles: Boolean): String;
begin
	dwShowFolderBrowserDialog(ParentWindow, DisplayText, '', Editable, NewStyle, ShowFiles);
end;

function dwShowFolderBrowserDialog(ParentWindow: HWND; const DisplayText, InitPath: String; const Editable, NewStyle, ShowFiles: Boolean): String;
var
	BI: TBrowseInfo;
   Flags: LongWord;
   SelectedPIDL: PItemIDList;

   procedure MergeFlag(var Flag: LongWord; const Value: LongWord);
   begin
   	Flag:= Flag or Value;
   end;

begin
   Flags:= 0;
   if Editable 	then MergeFlag(Flags, BIF_EDITBOX);
   if NewStyle 	then MergeFlag(Flags, BIF_NEWDIALOGSTYLE);
   if ShowFiles 	then MergeFlag(Flags, BIF_BROWSEINCLUDEFILES);
   MergeFlag(Flags, BIF_UAHINT);
   ZeroMemory(@BI, SizeOf(TBrowseInfo));
   with BI do begin
      hwndOwner:= ParentWindow;
//      pidlRoot:= NIL;
      GetMem(pszDisplayName, MAX_PATH);
      lpszTitle:= PChar(DisplayText);
      ulFlags:= Flags;
      dwInternal.dwiIndirectGlobalDummyString:= InitPath;
      lpfn:= dwInternal.BrowseForFolderCallBack;
      lParam := Longint(ParentWindow);
//      lParam:= 0;
      iImage:= 0;
   end;
   SelectedPIDL:= SHBrowseForFolder(bi);
   if SelectedPIDL <> NIL then begin
   	if SHGetPathFromIDList(SelectedPIDL, BI.pszDisplayName) then
      	result:= StrPas(BI.pszDisplayName);
   end;
   FreeMem(BI.pszDisplayName);
   GlobalFreePtr(SelectedPIDL);
end;

procedure dwFileOpenWithDialog(const Filename: String);
begin
  ShellExecute(GetCurrentProcess, 'open', PChar('rundll32.exe'),
    PChar('shell32.dll,OpenAs_RunDLL ' + FileName), nil, SW_SHOWNORMAL);
end;

procedure dwFilePropertiesDialog(const Filename: String);
var
  sei: TShellExecuteInfo;
begin
  FillChar(sei, SizeOf(sei), 0);
  sei.cbSize := SizeOf(sei);
  sei.lpFile := PChar(FileName);
  sei.lpVerb := 'properties';
  sei.fMask  := SEE_MASK_INVOKEIDLIST;
  ShellExecuteEx(@sei);
end;

procedure dwShowFormatDriveDialog(const Drive: Char; const Capacity: TDWFmtDriveDlgCapacity; const QuickErase: Boolean);
var
	CapFlag, QuickFlag: DWord;
begin
   QuickFlag:= 0;
   CapFlag:= 0;
	case Capacity of
   dwfddcDefault: 	CapFlag:= SHFMT_ID_DEFAULT;
   dwfddc1_44M:		CapFlag:= $6;
   dwfddc720K:			CapFlag:= $5;
   dwfddcMakeBoot:	CapFlag:= $0;
   end;
   if QuickErase then QuickFlag:= QuickFlag or SHFMT_OPT_FULL;
   if CapFlag = $0 then QuickFlag:= SHFMT_OPT_SYSONLY;
   dwiFormatDriveDlg(0, Ord(UpCase(Drive))-65, CapFlag, QuickFlag);
end;

procedure dwShowInteger(Int: Integer);
begin
   ShowMessage(IntToStr(Int));
end;

procedure dwShowPoint(Point: TPoint);
begin
	ShowMessage('Left: ' + IntToStr(Point.X) + #13#10 +
               'Top: ' + IntToStr(Point.Y));
end;

procedure dwShowRect(Rect: TRect);
begin
	ShowMessage('Left: ' + IntToStr(Rect.Left) + #13#10 +
               'Top: ' + IntToStr(Rect.Top) + #13#10 +
               'Right: ' + IntToStr(Rect.Right) + #13#10 +
               'Bottom: ' + IntToStr(Rect.Bottom) + #13#10 +
               'Height: ' + IntToStr(Rect.Bottom - Rect.Top) + #13#10 +
               'Width: ' + IntToStr(Rect.Right - Rect.Left));
end;

procedure dwShowSpecialDialog(const SpecialDialog: TDWWindowsDialog);
begin
   case SpecialDialog of
   dwwdCopyFloppy:
      begin
         ShellExecute(GetCurrentProcess,'open', 'rundll32.exe',
                      PChar('diskcopy,DiskCopyRunDll'), nil, SW_SHOWNORMAL);
      end;
   dwwdHardwareDetect:
      begin
         if (dwIsWin2K or dwIsWinXP) then
         	dwiOpenCPL('HDWWIZ.CPL')
         else
         	ShellExecute(GetCurrentProcess, 'open','rundll32.exe',
                         'sysdm.cpl,installdevice_rundll', nil, SW_SHOWNORMAL);
      end;
   dwwdControlPanel:
      begin
         ShellExecute(GetCurrentProcess, 'open','rundll32.exe',
                         'shell32,Control_RunDLL', nil, SW_SHOWNORMAL);
      end;
   dwwdRASWizard:
      begin
         ShellExecute(GetCurrentProcess, 'open','rundll32.exe',
                         'rnaui.dll,RnaWizard /1', nil, SW_SHOWNORMAL);
      end;
   dwwdWindowsShutdownDialog:
      begin
         PostMessage(FindWindow('Progman',nil), WM_CLOSE, 0, 0);
      end;
   dwwdExecute:
      begin
         keybd_event(VK_LWIN, MapVirtualKey(VK_LWIN, 0), 0, 0);
         keybd_event(Ord('R'), MapVirtualKey(Ord('R'), 0), 0, 0);
         keybd_event(Ord('R'), MapVirtualKey(Ord('R'), 0), KEYEVENTF_KEYUP, 0);
         keybd_event(VK_LWIN, MapVirtualKey(VK_LWIN, 0), KEYEVENTF_KEYUP, 0);
      end;
   dwwdFindFiles:
      begin
         keybd_event(VK_LWIN, MapVirtualKey(VK_LWIN, 0), 0, 0);
         keybd_event(Ord('F'), MapVirtualKey(Ord('F'), 0), 0, 0);
         keybd_event(Ord('F'), MapVirtualKey(Ord('F'), 0), KEYEVENTF_KEYUP, 0);
         keybd_event(VK_LWIN, MapVirtualKey(VK_LWIN, 0), KEYEVENTF_KEYUP, 0);
      end;
   dwwdStartMenu:
      begin
         keybd_event(VK_LWIN, MapVirtualKey(VK_LWIN, 0), 0, 0);
			keybd_event(VK_LWIN, MapVirtualKey(VK_LWIN, 0), KEYEVENTF_KEYUP, 0)
      end;
   dwwdTime:
      begin
         ShellExecute(GetCurrentProcess, 'open', 'control',
                        'date/time', nil, SW_SHOW)
      end;
   dwwdMouseProperties:
      begin
         dwiOpenCPL('MAIN.CPL @0');
      end;
   dwwdKeyboardProperties:
      begin
         dwiOpenCPL('MAIN.CPL @1');
      end;
   dwwdPrinterProperties:
      begin
         dwiOpenCPL('MAIN.CPL @2');
      end;
   dwwdFontsProperties:
      begin
         dwiOpenCPL('MAIN.CPL @3');
      end;
   dwwdSoftwareProperties:
      begin
         dwiOpenCPL('APPWIZ.CPL');
      end;
   dwwdDisplayProperties:
      begin
         dwiOpenCPL('DESK.CPL');
      end;
   dwwdInternetProperties:
      begin
         dwiOpenCPL('INETCPL.CPL');
      end;
   dwwdLocaleProperties:
      begin
         dwiOpenCPL('INTL.CPL');
      end;
   dwwdMultimediaProperties:
      begin
         dwiOpenCPL('MMSYS.CPL');
      end;
   dwwdModemProperties:
      begin
         dwiOpenCPL('MODEM.CPL');
      end;
   dwwdNetworkProperties:
      begin
         if (dwIsWin2K or dwIsWinXP) then
            dwiOpenCPL('NCPA.CPL')
         else
         	dwiOpenCPL('NETCPL.CPL');
      end;
   dwwdPasswordsProperties:
      begin
         dwiOpenCPL('PASSWORD.CPL');
      end;
   dwwdScanCamProperties:
      begin
         dwiOpenCPL('STICPL.CPL');
      end;
   dwwdSystemProperties:
      begin
         dwiOpenCPL('SYSDM.CPL');
      end;
   dwwdDialProperties:
      begin
         dwiOpenCPL('TELEPHON.CPL');
      end;
   dwwdGamecontrollerProperties:
      begin
         dwiOpenCPL('JOY.CPL');
      end;
   dwwdAccessibilityProperties:
      begin
         dwiOpenCPL('ACCESS.CPL');
      end;
   dwwdTweakUIProperties:
      begin
         dwiOpenCPL('TWEAKUI.CPL');
      end;
   dwwdODBCProperties:
      begin
         dwiOpenCPL('ODBCCP32.CPL');
      end;
   dwwdPowerManagementProperties:
      begin
         dwiOpenCPL('POWERCFG.CPL');
      end;
   dwwdThemesProperties:
      begin
         dwiOpenCPL('THEMES.CPL');
      end;
   dwwdBDEProperties:
      begin
         dwiOpenCPL('BDEADMIN.CPL');
      end;
   dwwdMailProperties:
      begin
         dwiOpenCPL('MLCFG32.CPL');
      end;
   dwwdRunFile:
   	begin
         dwiShowRunFileDialog(FindWindow('Shell_TrayWnd', NIL), NIL, NIL, NIL,
                              RFF_NOOPT, TRUE);
      end;
   dwwdRecycleBin:
   	begin
      	dwiOpenRecycleBin;
      end;
   dwwdScreenSaver:
   	begin
      	SendMessage(GetDesktopWindow, WM_SYSCOMMAND, SC_SCREENSAVE, 0);
      end;
   dwwdWindowsAboutDlg:
   	begin
      	dwiSHAbout(0, PChar(''), PChar(''), 0);
      end;
   dwwdUserManager:
   	begin
         if (dwIsWin2K or dwIsWinXP) then
         	ShellExecute(GetCurrentProcess, 'open', 'rundll32.exe',
                         'netplwiz.dll,UsersRunDll', nil, SW_SHOWNORMAL);
         if dwIsNT4 then
         	ShellExecute(0, 'open', 'musrmgr.exe', '', '', SW_SHOW);
      end;                                              
   end;
end;

procedure dwStartNewBrowserWindow(const URL: string);
var
  DDEConv: TDDEClientConv;
  URLFired: bool;
  App: string;
  UpApp: string;
  p: array[0..MAX_PATH] of Char;
begin
  UrlFired := False;
  App := dwGetAssociatedProgram('HTM');
  UpApp := Uppercase(App);
  Delete(App, Pos('.EXE', UpAPP), Length(App));
  if Pos('NETSCAPE.EXE',
    UpApp) > 0 then
  begin
    DDEConv := TDDEClientConv.Create(nil);
    DDEConv.ServiceApplication := App;
    if DDEConv.SetLink('NETSCAPE', 'WWW_OpenURL') then
      if DDEConv.RequestData(URL + ',,0x0,0x0') <> nil then
        if DDEConv.SetLink('NETSCAPE', 'WWW_Activate') then
          URLFired := DDEConv.RequestData('0xFFFFFFFF,0x0') <> nil;
    DDEConv.Free;
  end
  else if Pos('IEXPLORE.EXE',
    UpApp) > 0 then
  begin
    DDEConv := TDDEClientConv.Create(nil);
    DDEConv.ServiceApplication := App;
    if DDEConv.SetLink('iexplore', 'WWW_OpenURL') then
      if DDEConv.RequestData(URL + ',,0') <> nil then
        if DDEConv.SetLink('iexplore', 'WWW_Activate') then
          URLFired := DDEConv.RequestData('0,0') <> nil;
    DDEConv.Free;
  end;
  if UrlFired = False then
    WinExec(StrPCopy(@p, URL), SW_SHOWNORMAL);
end;

end.
