{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwFXSystem.pas
 Description: Functions for DirectX and OpenGL engines
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     15.12.2002
 Last mod.:   16.08.2004
 History:     n/a
-----------------------------------------------------------------------------}

unit dwFXSystem;

interface

uses dwTypes;

function dwGetDirectXName:         String;
function dwGetDirectXVersion:      TDWDirectXVersion;
function dwGetOpenGLData:          TDWOpenGLVersion;
function dwGetOpenGLDriver:        String;
function dwGetOpenGLName:          String;
function dwReadFullDirectXVersion: String;
function dwReadFullOpenGLVersion:  String;

implementation

uses dwConsts, dwWinSystem,
     Classes, Registry, SysUtils, Windows;

function dwReadFullDirectXVersion: String;
begin
   with TRegistry.Create do begin
      RootKey:= HKEY_LOCAL_MACHINE;
      OpenKey(DX_PATH, FALSE);
      result:= ReadString('Version');
      Free;
   end;
end;

function dwGetDirectXName: String;
var
  IntBuffer:    Integer;
  Buffer:       String;
begin
   with TRegistry.Create do begin
      RootKey := HKEY_LOCAL_MACHINE;
      OpenKey(DX_PATH, FALSE);
      Buffer := ReadString('Version');
      Free;
   end;
   with TStringList.Create do begin
      Text := StringReplace(Buffer, '.', #13, [rfReplaceAll]);
      Buffer := 'DirectX ';
      IntBuffer := StrToInt(Strings[1]);
      Buffer := Buffer + IntToStr(IntBuffer) + '.';
      IntBuffer := StrToInt(Strings[2]);
      Buffer := Buffer + IntToStr(IntBuffer);
      Free;
   end;
  result:= Buffer;
end;

function dwGetDirectXVersion: TDWDirectXVersion;
var
  StrBuffer:    String;
  Buffer:       TDWDirectXVersion;
begin
   with TRegistry.Create do begin
      RootKey := HKEY_LOCAL_MACHINE;
      OpenKey(DX_PATH, FALSE);
      StrBuffer:= ReadString('Version');
      Free;
   end;
   with TStringList.Create do begin
      Text := StringReplace(StrBuffer, '.', #13, [rfReplaceAll]);
      with Buffer do begin
         MajorVersion:=          StrToInt(Strings[1]);
         MinorVersion:=          StrToInt(Strings[2]);
         BuildVersion:=          StrToInt(Strings[3]);
         FullVersionAsString:=   Text;
      end;
      Free;
   end;
  result:= Buffer;
end;

function dwReadFullOpenGLVersion: String;
var
  reg_key:      String;
  Buffer:       String;
begin
   if dwIsNTMachine then begin
      reg_key:= StringReplace(GL_PATH,
                              '%OS_VERSION_DATA%',
                              'Windows NT',
                              [rfReplaceAll]);
   end else begin
      reg_key:= StringReplace(GL_PATH,
                              '%OS_VERSION_DATA%',
                              'Windows',
                              [rfReplaceAll]);
   end;
   with TRegistry.Create do begin
      RootKey:= HKEY_LOCAL_MACHINE;
      OpenKey(reg_key, FALSE);
      Buffer:= IntToStr(ReadInteger('Version'));
      Free;
   end;
   result:= Buffer;
end;

function dwGetOpenGLDriver: String;
var
  reg_key:      String;
  Buffer:       String;
begin
   if dwIsNTMachine then begin
      reg_key:= StringReplace(GL_PATH,
                              '%OS_VERSION_DATA%',
                              'Windows NT',
                              [rfReplaceAll]);
   end else begin
      reg_key:= StringReplace(GL_PATH,
                              '%OS_VERSION_DATA%',
                              'Windows',
                              [rfReplaceAll]);
   end;
   with TRegistry.Create do begin
      RootKey:= HKEY_LOCAL_MACHINE;
      OpenKey(reg_key, FALSE);
      Buffer:= ReadString('Dll') + '.dll';
      Free;
   end;
   result := Buffer;
end;

function dwGetOpenGLName: String;
var
  reg_key:      String;
  Buffer:       String;
begin
   if dwIsNTMachine then begin
      reg_key:= StringReplace(GL_PATH,
                              '%OS_VERSION_DATA%',
                              'Windows NT',
                              [rfReplaceAll]);
   end else begin
      reg_key:= StringReplace(GL_PATH,
                              '%OS_VERSION_DATA%',
                              'Windows',
                              [rfReplaceAll]);
   end;
   Buffer := 'OpenGL ';
   with TRegistry.Create do begin
      RootKey:= HKEY_LOCAL_MACHINE;
      OpenKey(reg_key, FALSE);
      Buffer := Buffer + IntToStr(ReadInteger('Version'));
      Free;
   end;
   result := Buffer;
end;

function dwGetOpenGLData: TDWOpenGLVersion;
var
  reg_key:      String;
  Buffer:       TDWOpenGLVersion;
begin
   if dwIsNTMachine then begin
      reg_key:= StringReplace(GL_PATH,
                              '%OS_VERSION_DATA%',
                              'Windows NT',
                              [rfReplaceAll]);
   end else begin
      reg_key:= StringReplace(GL_PATH,
                              '%OS_VERSION_DATA%',
                              'Windows',
                              [rfReplaceAll]);
   end;
   with TRegistry.Create do begin
      RootKey:= HKEY_LOCAL_MACHINE;
      OpenKey(reg_key, FALSE);
      with Buffer do begin
         GLVersion:=       IntToStr(ReadInteger('Version'));
         DriverVersion:=   IntToStr(ReadInteger('DriverVersion'));
         DriverNAME:=      ReadString('Dll') + '.dll';
      end;
      Free;
   end;
   result:= Buffer;
end;

end.

