{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwHardware.pas
 Description: Hardware detection functions
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     14.12.2002
 Last mod.:   16.08.2004
 History:     n/a
-----------------------------------------------------------------------------}
unit dwHardware;

interface

uses dwTypes,
     SysUtils, Windows;

function dwComPortAvailable(const COMPort: TDWComPort): Boolean;
function dwCPUClock: Double;
function dwGetProcessorFeatures: TDWProcessorFeatures;
function dwSoundCardInstalled: Boolean;

implementation

uses dwConsts, dwConvert, dwWinSystem,
     MMSystem;

function dwComPortAvailable(const COMPort: TDWComPort): Boolean;
var
  DeviceName: array[0..80] of Char;
  ComFile: THandle;
begin
  StrPCopy(DeviceName, dwComPortToStr(COMPort));
  ComFile := CreateFile(DeviceName, GENERIC_READ or GENERIC_WRITE, 0, nil,
    OPEN_EXISTING,
    FILE_ATTRIBUTE_NORMAL, 0);
  Result := ComFile <> INVALID_HANDLE_VALUE;
  CloseHandle(ComFile);
end;

function dwCPUClock: Double;
const
 DelayTime= 500;
var
  TimerHigh,TimerLow:DWORD;
begin
  SetPriorityClass( GetCurrentProcess,REALTIME_PRIORITY_CLASS);
  SetThreadPriority( GetCurrentThread,THREAD_PRIORITY_TIME_CRITICAL);
  Sleep( 10);
  asm
    dw 310Fh
    mov TimerLow,eax
    mov TimerHigh,edx
  end;
  Sleep( DelayTime);
  asm
    dw 310Fh
    sub eax,TimerLow
    sbb edx,TimerHigh
    mov TimerLow,eax
    mov TimerHigh,edx
  end;
  Result:=TimerLow / (1000.0*DelayTime);
end;

function dwGetProcessorFeatures: TDWProcessorFeatures;
var
   FloatPoint: Boolean;
begin
   if IsProcessorFeaturePresent(PF_FLOATING_POINT_PRECISION_ERRATA) then
      Include(result, dwpftFloatingError);
   FloatPoint:= IsProcessorFeaturePresent(PF_FLOATING_POINT_EMULATED);
   if dwIsNT4 then FloatPoint:= not FloatPoint; // inverse return bug in NT4
   if FloatPoint then Include(result, dwpftPointEmulated);
   if IsProcessorFeaturePresent(PF_COMPARE_EXCHANGE_DOUBLE) then
      Include(result, dwpftDoubleExchange);
   if IsProcessorFeaturePresent(PF_MMX_INSTRUCTIONS_AVAILABLE) then
      Include(result, dwpftMMX);
   if IsProcessorFeaturePresent(PF_XMMI_INSTRUCTIONS_AVAILABLE) then
      Include(result, dwpftSSE);
   if IsProcessorFeaturePresent(PF_3DNOW_INSTRUCTIONS_AVAILABLE) then
      Include(result, dwpft3DNow);
   if IsProcessorFeaturePresent(PF_RDTSC_INSTRUCTION_AVAILABLE) then
      Include(result, dwpftRDTSC);
   if IsProcessorFeaturePresent(PF_PAE_ENABLED) then
      Include(result, dwpftPAE);
   if IsProcessorFeaturePresent(PF_XMMI64_INSTRUCTIONS_AVAILABLE) then
      Include(result, dwpftSSE2);
end;

function dwSoundCardInstalled: Boolean;
begin
   result:= WaveOutGetNumDevs > 0;
end;

end.
 