{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwIniFiles.pas
 Description: AddOn to Delphi's IniFiles.pas
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     15.12.2002
 Last mod.:   16.08.2004
 History:     n/a
-----------------------------------------------------------------------------}

unit dwIniFiles;

interface

uses Classes, Forms, IniFiles;

function dwIniReadBool(Ini, Section, Value: String): Boolean;
function dwIniReadInteger(Ini, Section, Value: String): Integer;
function dwIniReadString(Ini, Section, Value: String): String;
function dwIniValueList(Ini, Section: String): TStringList;
function dwIniValueNameList(Ini, Section: String): TStringList;
function dwIniWriteBool(Ini, Section, Item: String; Value: Boolean): Boolean;
function dwIniWriteInteger(Ini, Section, Item: String; Value: Integer): Boolean;
function dwIniWriteString(Ini, Section, Item, Value: String): Boolean;
function dwIsNotNIL(Ini: TIniFile; Section, Item: String): Boolean;

procedure dwIniLoadForm(Form: TForm; IniFile: String);
procedure dwIniSaveForm(Form: TForm; IniFile: String);

implementation

uses dwConsts, dwConvert, dwStrings,
     Graphics, SysUtils;

function dwIniValueList(Ini, Section: String): TStringList;
var
   I: Integer;
   IniF: TIniFile;
   TempList: TStringList;
   TempStr: String;
begin
   result:= TStringList.Create;
   TempList:= TStringList.Create;
   if not FileExists(Ini) then exit;
   IniF:= TIniFile.Create(Ini);
   with IniF do begin
      if not SectionExists(Section) then exit;
      ReadSectionValues(Section, TempList);
      if TempList.Count = 0 then exit else if TempList.Count = 1 then begin
            TempStr:= TempList[0];
            result.Add(dwStrPart(TempStr, Pos('=', TempStr) + 1,
                               Length(TempStr)+1));
         end else
         if TempList.Count > 1 then
            for I:= 0 to TempList.Count - 1 do begin
               TempStr:= TempList[I];
               result.Add(dwStrPart(TempStr, Pos('=', TempStr) + 1,
                          Length(TempStr)+1));
            end;
   end;
   TempList.Free;
   IniF.Free;   
end;

function dwIniValueNameList(Ini, Section: String): TStringList;
var
   I: Integer;
   IniF: TIniFile;
   TempList: TStringList;
   TempStr: String;
begin
   result:= TStringList.Create;
   TempList:= TStringList.Create;
   if not FileExists(Ini) then exit;
   IniF:= TIniFile.Create(Ini);
   with IniF do begin
      if not SectionExists(Section) then exit;
      ReadSectionValues(Section, TempList);
      if TempList.Count = 0 then exit else if TempList.Count = 1 then
         result.Add(dwStrLeft(TempStr, Pos('=', TempStr) - 1)) else
         if TempList.Count > 1 then
            for I:= 0 to TempList.Count - 1 do begin
               TempStr:= TempList[I];
               result.Add(dwStrLeft(TempStr, Pos('=', TempStr) - 1));
            end;
   end;
   TempList.Free;
   IniF.Free;
end;

function dwIniReadInteger(Ini, Section, Value: String): Integer;
var IniF: TIniFile;
begin
   result:= 0;
   if not FileExists(Ini) then exit;
   IniF:= TIniFile.Create(Ini);
   if not IniF.SectionExists(Section) or not IniF.ValueExists(Section, Value)
      then exit;
   result:= IniF.ReadInteger(Section, Value, 0);
   IniF.Free;
end;

function dwIniReadBool(Ini, Section, Value: String): Boolean;
var IniF: TIniFile;
begin
   result:= FALSE;
   if not FileExists(Ini) then exit;
   IniF:= TIniFile.Create(Ini);
   if not IniF.SectionExists(Section) or not IniF.ValueExists(Section, Value)
      then exit;
   result:= IniF.ReadBool(Section, Value, FALSE);
   IniF.Free;
end;

function dwIniReadString(Ini, Section, Value: String): String;
var IniF: TIniFile;
begin
   result:= '';
//   if not FileExists(Ini) then exit;
   IniF:= TIniFile.Create(Ini);
//   if not IniF.SectionExists(Section) or not IniF.ValueExists(Section, Value)
//      then exit;
   result:= IniF.ReadString(Section, Value, '');
   IniF.Free;
end;

function dwIniWriteBool(Ini, Section, Item: String; Value: Boolean): Boolean;
var IniF: TIniFile;
begin
   result:= FALSE;
   if not FileExists(Ini) then exit;
   IniF:= TIniFile.Create(Ini);
   if not IniF.SectionExists(Section) or not IniF.ValueExists(Section, Item)
      then exit;
   IniF.WriteBool(Section, Item, Value);
   result:= IniF.ReadBool(Section, Item, FALSE);
   IniF.Free;
end;

function dwIniWriteString(Ini, Section, Item, Value: String): Boolean;
var IniF: TIniFile;
begin
//   result:= FALSE;
//   if not FileExists(Ini) then exit;
   IniF:= TIniFile.Create(Ini);
//   if not IniF.SectionExists(Section) or not IniF.ValueExists(Section, Item)
//      then exit;
   IniF.WriteString(Section, Item, Value);
   result:= IniF.ReadBool(Section, Item, FALSE);
   IniF.Free;
end;

function dwIniWriteInteger(Ini, Section, Item: String; Value: Integer): Boolean;
var IniF: TIniFile;
begin
//   result:= FALSE;
//   if not FileExists(Ini) then exit;
   IniF:= TIniFile.Create(Ini);
//   if not IniF.SectionExists(Section) or not IniF.ValueExists(Section, Item)
//      then exit;
   IniF.WriteInteger(Section, Item, Value);
   result:= IniF.ReadBool(Section, Item, FALSE);
   IniF.Free;
end;

procedure dwIniSaveForm(Form: TForm; IniFile: String);
var
   Ini: TIniFile;
   FN: String;
begin
   FN:= Form.Name;
   Ini:= TIniFile.Create(IniFile);
   with Ini do begin
      if Assigned( Form.Action) then
         WriteString(FN, 'Action', Form.Action.Name)
      else
         WriteString(FN, 'Action', NILSTRING);

      if Assigned( Form.ActiveControl) then
         WriteString(FN, 'ActiveControl', Form.ActiveControl.Name)
      else
         WriteString(FN, 'ActiveControl', NILSTRING);

      WriteString( FN,
                   'Align',
                   dwAlignToStr(Form.Align));

      WriteString( FN,
                   'Anchors',
                   dwAnchorsToStr(Form.Anchors));

      WriteBool(   FN,
                   'AutoScroll',
                   Form.AutoScroll);

      WriteBool(   FN,
                   'AutoSize',
                   Form.AutoSize);

      WriteString( FN,
                   'BiDiMode',
                   dwBiDiModeToStr(Form.BiDiMode));

      WriteString( FN,
                   'BorderIcons',
                   dwBorderIconsToStr(Form.BorderIcons));

      WriteString( FN,
                   'BorderStyle',
                   dwBorderStyleToStr(Form.BorderStyle));

      WriteInteger(FN,
                   'BorderWidth',
                   Form.BorderWidth);

      WriteString( FN,
                   'Caption',
                   Form.Caption);

      WriteInteger(FN,
                   'ClientHeight',
                   Form.ClientHeight);

      WriteInteger(FN,
                   'ClientWidth',
                   Form.ClientWidth);

      WriteString( FN,
                   'Color',
                   ColorToString(Form.Color));

      WriteString( FN,
                   'Constraints',
                   dwConstraintsToStr(Form.Constraints));

      WriteInteger(FN,
                   'Cursor',
                   Int64(Form.Cursor));

      WriteString( FN,
                   'DefaultMonitor',
                   dwDefaultMonitorToStr(Form.DefaultMonitor));

      WriteBool(   FN,
                   'DockSite',
                   Form.DockSite);

      WriteString( FN,
                   'DragKind',
                   dwDragKindToStr(Form.DragKind));

      WriteString( FN,
                   'DragMode',
                   dwDragModeToStr(Form.DragMode));

      WriteBool(   FN,
                   'Enabled',
                   Form.Enabled);

      WriteString( FN,
                   'Font',
                   dwFontToStr(Form.Font));

      WriteString( FN,
                   'FormStyle',
                   dwFormStyleToStr(Form.FormStyle));

      WriteInteger(FN,
                   'Height',
                   Form.Height);

      WriteInteger(FN,
                   'HelpContext',
                   Form.HelpContext);

      WriteString( FN,
                   'HelpFile',
                   Form.HelpFile);

      WriteString( FN,
                   'Hint',
                   Form.Hint);

      WriteString( FN,
                   'HorzScrollBar',
                   dwControlScrollBarToStr(Form.HorzScrollBar));

      if Assigned( Form.Icon) then
         WriteString(FN,
                     'Icon',
                     Form.Icon.ClassName)
      else
         WriteString(FN,
                     'Icon',
                     NILSTRING);

      WriteBool(   FN,
                   'KeyPreview',
                   Form.KeyPreview);

      WriteInteger(FN,
                   'Left',
                   Form.Left);

      if Assigned( Form.Menu) then
         WriteString(FN,
                     'Menu',
                     Form.Menu.Name)
      else
         WriteString(FN,
                     'Menu',
                     NILSTRING);

      WriteString( FN,
                   'Name',
                   Form.Name);

      if Assigned( Form.ObjectMenuItem) then
         WriteString(FN,
                     'ObjectMenuItem',
                     Form.ObjectMenuItem.Name)
      else
         WriteString(FN,
                     'ObjectMenuItem',
                     NILSTRING);

      WriteBool(   FN,
                   'ParentBiDiMode',
                   Form.ParentBiDiMode);

      WriteBool(   FN,
                   'ParentFont',
                   Form.ParentFont);

      WriteInteger(FN,
                   'PixelsPerInch',
                   Form.PixelsPerInch);

      if Assigned( Form.PopupMenu) then
         WriteString(FN,
                     'PopupMenu',
                     Form.PopupMenu.Name)
      else
         WriteString(FN,
                     'PopupMenu',
                     NILSTRING);

      WriteString( FN,
                   'Position',
                   dwPositionToStr(Form.Position));

      WriteString( FN,
                   'PrintScale',
                   dwPrintScaleToStr(Form.PrintScale));

      WriteBool(   FN,
                   'Scaled',
                   Form.Scaled);

      WriteBool(   FN,
                   'ShowHint',
                   Form.ShowHint);

      WriteInteger(FN,
                   'Tag',
                   Form.Tag);

      WriteInteger(FN,
                   'Top',
                   Form.Top);

      WriteBool(   FN,
                   'UseDockManager',
                   Form.UseDockManager);

      WriteString( FN,
                   'VertScrollBar',
                   dwControlScrollBarToStr(Form.VertScrollBar));

      WriteBool(   FN,
                   'Visible',
                   Form.Visible);

      WriteInteger(FN,
                   'Width',
                   Form.Width);

      if Assigned( Form.WindowMenu) then
         WriteString(FN,
                     'WindowMenu',
                     Form.WindowMenu.Name)
      else
         WriteString(FN,
                     'WindowMenu',
                     NILSTRING);

      WriteString( FN,
                   'WindowState',
                   dwWindowStateToStr(Form.WindowState));
   end;
end;

procedure dwIniLoadForm(Form: TForm; IniFile: String);
var
   Ini: TIniFile;
   FN: String;
begin
   Ini:= TIniFile.Create(IniFile);
   FN:= Form.Name;
   with Ini do begin
      if dwIsNotNIL(Ini, FN, 'Action') then
         Form.Action.Name:= ReadString(FN,
                                       'Action',
                                       NILSTRING);

      if dwIsNotNIL(Ini, FN, 'ActiveControl') then
         Form.ActiveControl.Name:= ReadString(FN,
                                              'ActiveControl',
                                              NILSTRING);

      Form.Align:=          dwStrToAlign(ReadString(FN,
                                                    'Align',
                                                    NILSTRING));

      Form.Anchors:=        dwStrToAnchors(ReadString(FN,
                                                      'Anchors',
                                                      NILSTRING));

      Form.AutoScroll:=     ReadBool(FN,
                                     'AutoScroll',
                                     TRUE);

      Form.AutoSize:=       ReadBool(FN,
                                     'AutoSize',
                                     TRUE);

      Form.BiDiMode:=       dwStrToBiDiMode(ReadString(FN,
                                                       'BiDiMode',
                                                       NILSTRING));

      Form.BorderIcons:=    dwStrToBorderIcons(ReadString(FN,
                                                          'BorderIcons',
                                                          NILSTRING));

      Form.BorderStyle:=    dwStrToBorderStyle(ReadString(FN,
                                                          'BorderStyle',
                                                          NILSTRING));

      Form.BorderWidth:=    ReadInteger(FN,
                                        'BorderWidth',
                                        0);

      Form.Caption:=        ReadString(FN,
                                       'Caption',
                                       Form.Name);

      Form.ClientHeight:=   ReadInteger(FN,
                                        'ClientHeight', Form.
                                        ClientHeight);

      Form.ClientWidth:=    ReadInteger(FN,
                                        'ClientWidth',
                                        Form.ClientWidth);

      Form.Color:=          StringToColor(ReadString(FN,
                                                     'Color',
                                                     ColorToString(Form.Color)));

      Form.Constraints:=    dwStrToConstraints(ReadString(FN,
                                                          'Constraints',
                                                          dwConstraintsToStr(Form.Constraints)));

      Form.Cursor:=         ReadInteger(FN,
                                        'Cursor',
                                        Int64(Form.Cursor));

      Form.DefaultMonitor:= dwStrToDefaultMonitor(ReadString(FN,
                                                             'DefaultMonitor',
                                                             dwDefaultMonitorToStr(Form.DefaultMonitor)));
      Form.DockSite:=       ReadBool(FN,
                                     'DockSite',
                                     Form.DockSite);

      Form.DragKind:=       dwStrToDragKind(ReadString(FN,
                                                       'DragKind',
                                                       dwDragKindToStr(Form.DragKind)));

      Form.DragMode:=       dwStrToDragMode(ReadString(FN,
                                                       'DragMode',
                                                       dwDragModeToStr(Form.DragMode)));

      Form.Enabled:=        ReadBool(FN,
                                     'Enabled',
                                     Form.Enabled);

      Form.Font:=           dwStrToFont(ReadString(FN,
                                                   'Font',
                                                   dwFontToStr(Form.Font)));

      Form.FormStyle:=      dwStrToFormStyle(ReadString(FN,
                                                        'FormStyle',
                                                        dwFormStyleToStr(Form.FormStyle)));

      Form.Height:=         ReadInteger(FN,
                                        'Height',
                                        Form.Height);

      Form.HelpContext:=    ReadInteger(FN,
                                        'HelpContext',
                                        Form.HelpContext);

      Form.HelpFile:=       ReadString(FN,
                                       'HelpFile',
                                       Form.HelpFile);

      Form.Hint:=           ReadString(FN,
                                       'Hint',
                                       Form.Hint);

      Form.HorzScrollBar:=  dwStrToControlScrollBar(ReadString(FN,
                                                               'HorzScrollBar',
                                                               ''{ControlScrollBarToStr(Form.HorzScrollBar)}));
//      if IsNotNIL(Ini, FN, 'Icon') then Form.Icon.ClassName:= ReadString(FN, 'Icon', Form.Icon.ClassName));

      Form.KeyPreview:=     ReadBool(FN,
                                     'KeyPreview',
                                     Form.KeyPreview);

      Form.Left:=           ReadInteger(FN,
                                        'Left',
                                        Form.Left);

      if dwIsNotNIL(Ini, FN, 'Menu') then
         Form.Menu.Name:= ReadString(FN,
                                     'Menu',
                                     Form.Menu.Name);

      Form.Name:=           ReadString(FN,
                                       'Name',
                                       Form.Name);

      if dwIsNotNIL(Ini, FN, 'ObjectMenuItem') then
         Form.ObjectMenuItem.Name:= ReadString(FN,
                                               'ObjectMenuItem',
                                               Form.ObjectMenuItem.Name);

      Form.ParentBiDiMode:= ReadBool(FN,
                                     'ParentBiDiMode',
                                     Form.ParentBiDiMode);

      Form.ParentFont:=     ReadBool(FN,
                                     'ParentFont',
                                     Form.ParentFont);

      Form.PixelsPerInch:=  ReadInteger(FN,
                                        'PixelsPerInch',
                                        Form.PixelsPerInch);

      if dwIsNotNIL(Ini, FN, 'PopupMenu') then
         Form.PopupMenu.Name:= ReadString(FN,
                                          'PopupMenu',
                                          Form.PopupMenu.Name);

      Form.Position:=       dwStrToPosition(ReadString(FN,
                                                       'Position',
                                                       dwPositionToStr(Form.Position)));

      Form.PrintScale:=     dwStrToPrintScale(ReadString(FN,
                                                         'PrintScale',
                                                         dwPrintScaleToStr(Form.PrintScale)));

      Form.Scaled:=         ReadBool(FN,
                                     'Scaled',
                                     Form.Scaled);

      Form.ShowHint:=       ReadBool(FN,
                                     'ShowHint',
                                     Form.ShowHint);

      Form.Tag:=            ReadInteger(FN,
                                        'Tag',
                                        Form.Tag);

      Form.Top:=            ReadInteger(FN,
                                        'Top',
                                        Form.Top);

      Form.UseDockManager:= ReadBool(FN,
                                     'UseDockManager',
                                     Form.UseDockManager);

//      Form.VertScrollBar:= StrToControlScrollBar(ReadString(FN, 'VertScrollBar', ControlScrollBarToStr(Form.VertScrollBar)));

      Form.Visible:=        ReadBool(FN,
                                     'Visible',
                                     Form.Visible);

      Form.Width:=          ReadInteger(FN,
                                        'Width',
                                        Form.Width);

      if dwIsNotNIL(Ini, FN, 'WindowMenu') then
         Form.WindowMenu.Name:= ReadString(FN,
                                           'WindowMenu',
                                           Form.WindowMenu.Name);

      Form.WindowState:=    dwStrToWindowState(ReadString(FN,
                                                          'WindowState',
                                                          dwWindowStateToStr(Form.WindowState)));
   end;
end;

function dwIsNotNIL(Ini: TIniFile; Section, Item: String): Boolean;
begin
   result:= not (Ini.ReadString(Section, Item, NILSTRING) = NILSTRING);
end;


end.
