{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwMathematics.pas
 Description: Mathematic and numeric functions
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     14.12.2002
 Last mod.:   16.08.2004
 History:     n/a
-----------------------------------------------------------------------------}
unit dwMathematics;

interface

uses Windows;

function dwBitIsSet(w : DWord; Bitnr:integer):Boolean;
function dwCountBits(const Value: Integer): Integer;
function dwDiff(const Value1, Value2: Integer): Integer;
function dwExpon(const Value, Exponent: Integer): Integer;
function dwExtractBits(const Value, Start, Count: Integer): Integer;
function dwFactorial(const Value: Word): LongInt;
function dwIsPrimeNumber(Number: Cardinal): Boolean;
function dwRndBetween(const RangeMinor, RangeMajor: Integer): Integer;
function dwRoundDown(X: Real): Integer;
function dwRoundUp(X: Real): Integer;
function dwSimpleChecksum(const Str: String): Integer;
function dwValidateRoman(const Roman: String): Boolean;
function dwValidateValueForFreeNotation(const Value, NotationConfig: String): Boolean;

procedure dwResetBit(var w : DWord; Bitnr:integer);
procedure dwSetBit(var w : DWord; Bitnr:integer);

implementation

function dwBitIsSet(w : DWord; Bitnr:integer):Boolean;
begin
   {basiert auf WernerSt, Spotlight Delphi Forum, 17.12.2001}
   result:=(w and (1 shl Bitnr))<>0;
end;

function dwCountBits(const Value: Integer): Integer;
asm
  {basiert auf Guido Gybels, http://www.optimalcode.com/Guido/basmex6.html}
  mov ecx,eax
  xor eax,eax
  test ecx,ecx
  jz @@ending
 @@counting:
  shr ecx,1
  adc eax,0
  test ecx,ecx
  jnz @@counting
 @@ending:
end;

function dwDiff(const Value1, Value2: Integer): Integer;
begin
   if (Value1 > Value2) then
      result:= Value1 - Value2
   else
      result:= Value2 - Value1;
end;

function dwExpon(const Value, Exponent: Integer): Integer;
var i: Integer;
    wert: Integer;
begin
  wert:=Value;
  if Exponent=0 then
    begin
    result:=1;
    exit;
    end
  else
    begin
    for i:=1 to Exponent-1 do
      begin
      wert:=wert*Value;
      end; {for}
    result:=wert;
    end;
end;

function dwExtractBits(const Value, Start, Count: Integer): Integer;
const
  {basiert auf Guido Gybels, http://www.optimalcode.com/Guido/basmex6.html}
  Mask: array[0..31] of Int64 =
            ($01,$03,$07,$0F,$1F,$3F,$7F,$FF,
            $01FF,$03FF,$07FF,$0FFF,$1FFF,$3FFF,$7FFF,$FFFF,
            $01FFFF,$03FFFF,$07FFFF,$0FFFFF,
            $1FFFFF,$3FFFFF,$7FFFFF,$FFFFFF,
            $01FFFFFF,$03FFFFFF,$07FFFFFF,$0FFFFFFF,
            $1FFFFFFF,$3FFFFFFF,$7FFFFFFF,$FFFFFFFF);
asm
  xchg ecx,edx
  test edx,edx
  jnz @@isoke
  xor eax,eax
  jmp @@ending
 @@isoke:
  dec edx
  and edx,31
  shr eax,cl
  and eax,dword ptr [Mask+edx*4]
 @@ending:
end;

function dwFactorial(const Value: Word): LongInt;
begin
  if Value > 1 then
    Result := Value * dwFactorial(Value - 1)
  else
    Result := 1;
end;

function dwIsPrimeNumber(Number: Cardinal): Boolean;
var
  Z: Real;
  Max: LongInt;
  Divisor: LongInt;
begin
  result:= FALSE;
  if (Number and 1) = 0 then exit;
  Z       := Sqrt(Number);
  Max     := Trunc(Z) + 1;
  Divisor := 3;
  while Max > Divisor do
  begin
    if (Number mod Divisor) = 0 then Exit;
    Inc(Divisor, 2);
    if (Number mod Divisor) = 0 then Exit;
    Inc(Divisor, 4);
  end;
  result:= TRUE;
end;

function dwRndBetween(const RangeMinor, RangeMajor: Integer): Integer;
var
   Rnd: Integer;
begin
	result:= 0;
	Randomize;
   Rnd:= RangeMinor + Random(RangeMajor - RangeMinor);
   result:= Rnd;
end;

function dwRoundDown(X: Real): Integer;
begin
   if Trunc(X)<>X then begin
      result:=Trunc(X)-1;
      exit;
   end else result:= Trunc(X);
end;

function dwRoundUp(X: Real): Integer;
begin
   if Trunc(X)<>X then begin
      result:=Trunc(X)+1;
      exit;
   end else result:= Trunc(X);
end;

function dwSimpleChecksum(const Str: String): Integer;
var
   I, Value: Integer;
   Chk: Boolean;
begin
   Chk:= FALSE;
   result:= 0;
   Value:= 0;
   if Length(Str) < 1 then exit;
   for I:= 1 to Length(Str) do begin
      if Chk then Value:= Value + (Ord(Str[I]) * 7) else
                  Value:= Value + (Ord(Str[I]) * 3);
      Chk:= not Chk;
   end;
   result:= Value;
end;

function dwValidateRoman(const Roman: String): Boolean;
const
   Romanos : Array[1..13] of String  =  ('M', 'CM', 'D', 'CD', 'C', 'XC', 'L',
                                         'XL', 'X', 'IX', 'V', 'IV', 'I');
 var
   i  : Integer;
   Ro : String;
   Longitud  : Integer;
   Temporal  : String;
   RomanTemp : String;
   Cantidad  : Integer;
   NumMayor  : Boolean;
   Entrado   : Boolean;
   Sucesion  : Boolean;
 begin
   Result    := False;
   Temporal  := '';
   NumMayor  := True;
   Entrado   := False;
   Sucesion  := True;

   RomanTemp := Roman;

   for i := 1 to 13 do
   begin
     Longitud := Length(Romanos[i]);
     Ro := Copy(RomanTemp, 1, Longitud);
     Cantidad := 0;


     while ((Length(RomanTemp)>0) and (Ro = Romanos[i])) do
     begin
       RomanTemp  := Copy(RomanTemp, 1 + Longitud, Length(RomanTemp) - Longitud);
       Temporal := Temporal + Romanos[i];
       Ro := Copy(RomanTemp, 1, Longitud);
       Inc(Cantidad);
       Entrado := True;
     end;

     if Entrado then
     begin
       case i of
         5, 9, 13:
           if Cantidad > 3 then
             NumMayor := False;
         2, 3, 4, 6, 7, 8, 10, 11, 12:
           if Cantidad > 1 then
             NumMayor := False;
       end;
       Entrado := False;
     end;
   end;

   if ((Temporal = Roman) and NumMayor) then
   begin
     RomanTemp := Roman;
     for i := 1 to 13 do
     begin
       Longitud := Length(Romanos[i]);
       Ro := Copy(RomanTemp, 1, Longitud);
       while (Sucesion and (Length(RomanTemp)>0) and (Ro = Romanos[i])) do
       begin
         case i of
           2, 6, 10 :
           begin
             if (Copy(RomanTemp,3,1) = Romanos[i+1]) then
               Sucesion := False
             else
               if (Copy(RomanTemp,3,2) = Romanos[i+2]) then
                 Sucesion := False
               else
                 if (Copy(RomanTemp,3,1) = Romanos[i+3]) then
                   Sucesion := False;
           end;
           3, 7, 11:
             if (Copy(RomanTemp,2,2) = Romanos[i+1]) then
               Sucesion := False;
           4, 8, 12:
             if (Copy(RomanTemp,3,1) = Romanos[i+1]) then
               Sucesion := False
         end;
         RomanTemp := Copy(RomanTemp, 1 + Longitud, Length(RomanTemp) - Longitud);
         Ro := Copy(RomanTemp, 1, Longitud);
       end;
     end;
     Result := Sucesion;
   end
   else
     Result := False;
 end;

function dwValidateValueForFreeNotation(const Value, NotationConfig: String): Boolean;
var
   I: Integer;
begin
   result:= FALSE;
   for I:= 1 to Length(Value) do begin
      result:= (Pos(Value[I], NotationConfig) <> 0);
      if not result then Break;
   end;
end;

{-----------------------------------------------------------------------------
 Procedures
-----------------------------------------------------------------------------}
procedure dwResetBit(var w : DWord; Bitnr:integer);
begin
   {basiert auf WernerSt, Spotlight Delphi Forum, 17.12.2001}
   w:=w and ($FFFFFFFF xor (1 shl Bitnr));
end;

procedure dwSetBit(var w : DWord; Bitnr:integer);
begin
   {basiert auf WernerSt, Spotlight Delphi Forum, 17.12.2001}
   w:=w or (1 shl Bitnr);
end;

end.
