{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwPowerMgmt.pas
 Description: Functions for Power Management
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     14.12.2002
 Last mod.:   16.08.2004
 History:     n/a
-----------------------------------------------------------------------------}


unit dwPowerMgmt;

interface

uses dwTypes,
     Windows;

function dwGetACLineStatus: TDWACLineStatus;
function dwGetBatteryFullLifeTime: Integer;
function dwGetBatteryLifePercent: Byte;
function dwGetBatteryLifeTime: Integer;
function dwGetBatteryState: TDWBatteryStates;
function dwHibernateSupported: Boolean;
function dwSetSuspendState(const Hibernate, ForceCritical, DisableWakeEvent: Boolean): Boolean;
function dwSuspendSupported: Boolean;

var
	fcnSetSuspendState: function (Hibernate, ForceCritical, DisableWakeEvent: BOOL): BOOL  stdcall = nil;

implementation

uses dwInternal;

function dwGetACLineStatus: TDWACLineStatus;
var
   Sps: TSystemPowerStatus;
begin
   GetSystemPowerStatus(Sps);
   case Sps.ACLineStatus of
   0:   result:= dwalsOffline;
   1:   result:= dwalsOnline;
   else result:= dwalsUnknownState;
   end;
end;

function dwGetBatteryFullLifeTime: Integer;
var
   Sps: TSystemPowerStatus;
begin
   GetSystemPowerStatus(Sps);
   result:= Sps.BatteryFullLifeTime;
end;

function dwGetBatteryLifePercent: Byte;
var
   Sps: TSystemPowerStatus;
begin
   GetSystemPowerStatus(Sps);
   result:= Sps.BatteryLifePercent;
end;

function dwGetBatteryLifeTime: Integer;
var
   Sps: TSystemPowerStatus;
begin
   GetSystemPowerStatus(Sps);
   result:= Sps.BatteryLifeTime;
end;

function dwGetBatteryState: TDWBatteryStates;
var
   Sps: TSystemPowerStatus;
begin
   GetSystemPowerStatus(Sps);
   with Sps do begin
      if ((BatteryFlag and 1) = 1) then Include(result, dwbtsHigh);
      if ((BatteryFlag and 2) = 2) then Include(result, dwbtsLow);
      if ((BatteryFlag and 4) = 4) then Include(result, dwbtsCritical);
      if ((BatteryFlag and 8) = 8) then Include(result, dwbtsCharging);
      if ((BatteryFlag and 128) = 128) then Include(result, dwbtsNoSystemBattery);
      if ((BatteryFlag and 255) = 255) then Include(result, dwbtsUnknownState);
   end;
end;

function dwHibernateSupported: Boolean;
type
  TIsPwrHibernateAllowed = function: Boolean; stdcall;
var
  hPowrprof: HMODULE;
  IsPwrHibernateAllowed: TIsPwrHibernateAllowed;
begin
  Result := FALSE;
  if dwIsNT4 or dwIsWin95 then exit;
  hPowrprof := LoadLibrary('powrprof.dll');
  if hPowrprof <> 0 then
  begin
    try
      @IsPwrHibernateAllowed := GetProcAddress(hPowrprof, 'IsPwrHibernateAllowed');
      if @IsPwrHibernateAllowed <> NIL then
      begin
        Result := IsPwrHibernateAllowed;
      end;
    finally
      FreeLibrary(hPowrprof);
    end;
  end;
end;

function dwSetSuspendState(const Hibernate, ForceCritical, DisableWakeEvent: Boolean): Boolean;
begin
  if not Assigned(fcnSetSuspendState) then
    @fcnSetSuspendState := dwiLinkAPI('POWRPROF.dll', 'SetSuspendState');
  if Assigned(fcnSetSuspendState) then
    Result := fcnSetSuspendState(Hibernate, ForceCritical,
      DisableWakeEvent)
  else
    Result := False;
end;


type
  TIsPwrSuspendAllowed = function: Boolean;
  stdcall;
var
  hPowrprof: HMODULE;
  IsPwrSuspendAllowed: TIsPwrSuspendAllowed;
begin
  Result := False;
  hPowrprof := LoadLibrary('powrprof.dll');
  if hPowrprof <> 0 then
  begin
    try
      @IsPwrSuspendAllowed := GetProcAddress(hPowrprof, 'IsPwrSuspendAllowed');
      if @IsPwrSuspendAllowed <> nil then
      begin
        Result := IsPwrSuspendAllowed;
      end;
    finally
      FreeLibrary(hPowrprof);
    end;
  end;
end;

end.
