{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwPrinters.pas
 Description: Functions for printers
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     15.12.2002
 Last mod.:   16.08.2004
 History:     n/a
-----------------------------------------------------------------------------}

unit dwPrinters;

interface

function dwGetDefaultPaperBin: Integer;
function dwGetPrinterStrFromIndex(index : integer) : string;
function dwPRN_MMToPixelX(Millimeters: Integer): Integer;
function dwPRN_MMToPixelY(Millimeters: Integer): Integer;
function dwPRN_PXLPERMM_X(MM: Integer): Integer;
function dwPRN_PXLPERMM_Y(MM: Integer): Integer;

procedure dwSendASCII(Str, JobName: string);
procedure dwSetDefaultPrinter(NewDefPrinter : string);

implementation

uses dwDialogs,
     Messages, Printers, SysUtils, Windows, WinSpool;

function dwGetDefaultPaperBin: Integer;
var
  Device: array[0..cchDevicename-1] of Char;
  Driver: array[0..(MAX_PATH)-1] of Char;
  Port: array[0..32] of Char;
  hDMode: THandle;
  pDMode: PDevMode;
begin
  result:= -1; //Errorcode 
  Printer.GetPrinter(Device, Driver, Port, hDMode);
  if (hDMode<>0) then
  begin
    pDMode:=GlobalLock(hDMode);
    if pDMode<>nil then result:= pDMode^.dmDefaultSource;
  end;
end;

function dwGetPrinterStrFromIndex(index : integer) : string;
var
   pri : integer;
   hDeviceMode : THandle;
   Device,Driver,Port : array[0..255] of char;
   s,ResStr : array[0..255] of char;
begin
   pri:=Printer.PrinterIndex;
   Printer.PrinterIndex:=index;
   Printer.GetPrinter(Device,Driver,Port,hDeviceMode);
   GetProfileString('Devices',Device,'',ResStr,255);
   StrCopy (s, Device);
   StrCat (s, ',');
   StrCat (s, ResStr);
   Result:=String(s);
   Printer.PrinterIndex:=pri;
end;

procedure dwSetDefaultPrinter(NewDefPrinter : string);
var
   ResStr : array[0..255] of char;
begin
   StrPCopy(ResStr,NewdefPrinter);
   WriteProfileString ('windows', 'device', ResStr);
   StrCopy (ResStr, 'windows');
   SendMessage (HWND_BROADCAST, WM_WININICHANGE, 0, LongInt (@ResStr));
end;

function dwPRN_PXLPERMM_X(MM: Integer): Integer;
var
   xPelsPerInch, xPelsPerMM, xPelsOffset: Extended;
begin
   xPelsPerInch:= GetDeviceCaps(Printer.Canvas.Handle, LOGPIXELSX);
   xPelsPerMM:= (xPelsPerInch / 25.4);
   xPelsOffset:= GetDeviceCaps(Printer.Canvas.Handle, PHYSICALOFFSETX);
   result:= Abs(Round(MM * xPelsPerMM - xPelsOffset));
end;

function dwPRN_PXLPERMM_Y(MM: Integer): Integer;
var
   yPelsPerInch, yPelsPerMM, yPelsOffset: Extended;
begin
   yPelsPerInch:= GetDeviceCaps(Printer.Canvas.Handle, LOGPIXELSY);
   dwShowInteger(Round(yPelsPerInch));
   yPelsPerMM:= (yPelsPerInch / 25.4);
   yPelsOffset:= GetDeviceCaps(Printer.Canvas.Handle, PHYSICALOFFSETY);
   result:= Abs(Round(MM * yPelsPerMM - yPelsOffset));
end;

function dwPRN_MMToPixelX(Millimeters: Integer): Integer;
begin
   result:= dwPRN_PXLPERMM_X(Millimeters);
end;

function dwPRN_MMToPixelY(Millimeters: Integer): Integer;
begin
   result:= dwPRN_PXLPERMM_Y(Millimeters);
end;

procedure dwSendASCII(Str, JobName: string);
var Printer : array [0..255] of char;
    p       : Integer;
  function RawDataToPrinter(const szPrinterName : string;
                            const data:string;
                            dwCount : DWORD) : boolean;
  var
    hPrinter       : THandle;
    DocInfo        : TDocInfo1;
    dwJOB          : DWORD;
    dwBytesWritten : DWORD;
  begin
    Result := False;
    if OpenPrinter (pchar (szPrinterName), hPrinter, Nil) then
      try
        DocInfo.pDocName := PChar(JobName);
        DocInfo.pOutputFile := Nil;
        DocInfo.pDatatype := 'RAW';
        dwJob := StartDocPrinter (hPrinter, 1, @docInfo);
        if dwJob <> 0 then
          try
            if StartPagePrinter (hPrinter) then
              try
                if WritePrinter (hPrinter, Pchar(data), dwCount, dwBytesWritten) then
                  Result := dwBytesWritten = dwCount;
              finally
                EndPagePrinter (hPrinter)
              end
          finally
            EndDocPrinter (hPrinter);
          end
      finally
        ClosePrinter (hPrinter)
      end
  end;
begin
  GetProfileString ('windows',
                    'device',
                    ',,,',
                    Printer,
                    sizeof(Printer));
  p := Pos (',', Printer);
  if p > 0 then
    Printer [p - 1] := #0;
  RawDataToPrinter (Printer, Str, length(Str));
end;

end.
