{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwRegBase.pas
 Description: AddOn for DelphiWorks dwRegistry.pas
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     15.12.2002
 Last mod.:   16.08.2004
 History:     n/a
-----------------------------------------------------------------------------}
unit dwRegBase;

interface

uses Classes, Windows;

function dwOpenRegistry(const RootKey: HKEY; const Key: String; const AccessMode: LongWord; var RegHandle: HKEY): Boolean;
function dwCloseRegistry(var RegHandle: HKEY): Boolean;

function dwRegCreateKey(var RegHandle: HKEY; const NewKey: String; const AutoOpen: Boolean): Boolean; overload;
function dwRegCreateKey(const RootKey: HKEY; const Key, NewKey: String): Boolean; overload;

function dwRegKeyExists(var RegHandle: HKEY; const Key: String): Boolean; overload;
function dwRegKeyExists(const RootKey: HKEY; const Key: String): Boolean; overload;

function dwRegValueExists(var RegHandle: HKEY; const ValueName: String): Boolean; overload;
function dwRegValueExists(const RootKey: HKEY; const Key, ValueName: String): Boolean; overload;

function dwRegDeleteValue(var RegHandle: HKEY; const ValueName: String): Boolean; overload;
function dwRegDeleteValue(const RootKey: HKEY; const Key, ValueName: String): Boolean; overload;

function dwRegListSubKeys(var RegHandle: HKEY; List: TStrings): Boolean; overload;
function dwRegListSubKeys(const RootKey: HKEY; const Key: String; List: TStrings): Boolean; overload;

implementation

uses dwConsts;

procedure ValidateKey(var Key: String);
begin
	if Length(Key)>0 then if Key[1] <> '\' then Key:= '\' + Key;
end;

function dwOpenRegistry(const RootKey: HKEY; const Key: String; const AccessMode: LongWord; var RegHandle: HKEY): Boolean;
var
   sKey: String;
	pcKey: PChar;
begin
   sKey:= Key;
	ValidateKey(sKey);
   pcKey:= PChar(sKey);
   try
		result:= (RegOpenKeyEx(RootKey, @pcKey[1],
                             0,
                             AccessMode,
                             RegHandle) = ERROR_SUCCESS);
   except
   	RegCloseKey(RegHandle);
      result:= FALSE;
   end;
end;

function dwCloseRegistry(var RegHandle: HKEY): Boolean;
begin
	result:= (RegCloseKey(RegHandle) = ERROR_SUCCESS);
end;

function dwRegCreateKey(var RegHandle: HKEY; const NewKey: String; const AutoOpen: Boolean): Boolean; overload;
var
	NewRegHandle: HKEY;
   sKey: String;
begin
   if NewKey[1] = '\' then
   	sKey:= Copy(NewKey, 2, Length(NewKey))
   else
   	sKey:= NewKey;
	result:= (RegCreateKey(RegHandle, PChar(sKey), NewRegHandle) = ERROR_SUCCESS);
   if result and AutoOpen then begin
		dwCloseRegistry(RegHandle);
      RegHandle:= NewRegHandle;
   end;
end;

function dwRegCreateKey(const RootKey: HKEY; const Key, NewKey: String): Boolean; overload;
var
	RegHandle: HKEY;
begin
    result:= dwOpenRegistry(RootKey, Key, KEY_CREATE_SUB_KEY, RegHandle);
    result:= result and dwRegCreateKey(RegHandle, NewKey, FALSE);
    result:= result and dwCloseRegistry(RegHandle);
end;

function dwRegValueExists(var RegHandle: HKEY; const ValueName: String): Boolean; overload;
var
	ValueType: DWord;
begin
   ValueType:= REG_NONE;
	result:= (RegQueryValueEx(RegHandle, PChar(ValueName), NIL, @ValueType, NIL, NIL) = ERROR_SUCCESS) and (ValueType in REG_ANY);
end;

function dwRegValueExists(const RootKey: HKEY; const Key, ValueName: String): Boolean; overload;
var
	RegHandle: HKEY;
   sKey: String;
begin
   sKey:= Key;
   ValidateKey(sKey);
   ZeroMemory(@RegHandle, SizeOf(HKEY));
   result:= dwOpenRegistry(RootKey, PChar(sKey), KEY_READ, RegHandle);
   result:= result and dwRegValueExists(RegHandle, ValueName);
   result:= result and dwCloseRegistry(RegHandle);
end;

function dwRegDeleteValue(var RegHandle: HKEY; const ValueName: String): Boolean; overload;
begin
	result:= FALSE;
   if dwRegValueExists(RegHandle, ValueName) then begin
   	result:= (RegDeleteValue(RegHandle, PChar(ValueName)) = ERROR_SUCCESS);
   end;
end;

function dwRegKeyExists(var RegHandle: HKEY; const Key: String): Boolean; overload;
begin
//
end;

function dwRegKeyExists(const RootKey: HKEY; const Key: String): Boolean; overload;
begin
   //
end

function dwRegDeleteValue(const RootKey: HKEY; const Key, ValueName: String): Boolean; overload;
var
	RegHandle: HKEY;
begin
	result:= dwOpenRegistry(RootKey, Key, KEY_ALL_ACCESS, RegHandle);
   result:= result and dwRegDeleteValue(RegHandle, ValueName);
   result:= result and dwCloseRegistry(RegHandle);
end;

function dwRegListSubKeys(var RegHandle: HKEY; List: TStrings): Boolean; overload;
var
	Buffer: array[0..MAX_PATH] of Char;
   I: Integer;
   MaxItemLength: DWord;
   RetCode: DWord;
begin
   result:= FALSE;
   if not Assigned(List) then exit;
   List.Clear;
	I:= 0;
   while TRUE do begin
      ZeroMemory(@Buffer, SizeOf(Buffer));
      MaxItemLength:= SizeOf(Buffer);
   	RetCode:= RegEnumKeyEx(RegHandle,
                             I,
                             Buffer,
                             MaxItemLength,
                             NIL,
                             NIL,
                             NIL,
                             NIL);
      if RetCode = ERROR_SUCCESS then begin
      	List.Add(Buffer);
         Inc(I);
      end else if RetCode = ERROR_NO_MORE_ITEMS then begin
      	result:= TRUE;
         Break;
      end else begin
      	Break;
      end;
   end;
end;

function dwRegListSubKeys(const RootKey: HKEY; const Key: String; List: TStrings): Boolean; overload;
var
	RegHandle: HKEY;
   sKey: String;
begin
   sKey:= Key;
   ValidateKey(sKey);
   ZeroMemory(@RegHandle, SizeOf(HKEY));
   result:= dwOpenRegistry(RootKey, PChar(Key), KEY_READ, RegHandle);
   result:= result and dwRegListSubKeys(RegHandle, List);
   result:= result and dwCloseRegistry(RegHandle);
end;


end.
