{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwRegistry.pas
 Description: AddOn for Delphi's Registry.pas
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     15.12.2002
 Last mod.:   16.08.2004
 History:     n/a 
-----------------------------------------------------------------------------}

unit dwRegistry;

interface

uses dwTypes,
     Graphics, Windows;

function dwAutostartIsRegistered(const AutorunRootKey: TDWAutorunRootKey; const AutorunOption:  TDWAutorunOption; const ValueName: String): Boolean; overload;
function dwAutostartIsRegistered(const AutorunRootKey: TDWAutorunRootKey; const AutorunOption:  TDWAutorunOption; const ValueName, CommandString: String): Boolean; overload;
function dwGetAutostart(const AutorunRootKey: TDWAutorunRootKey; const AutorunOption: TDWAutorunOption; const ValueName: String): String;
function dwReadRegColor(const RootKey: HKEY; const KeyName, ValueName: String): TColor;
function dwRefreshIconCache: Boolean;
function dwRegisterAutostart(const AutorunRootKey: TDWAutorunRootKey; const AutorunOption: TDWAutorunOption; const ValueName, CommandString: String; const OverwriteExisting: Boolean): Boolean;
function dwUnregisterAutostart(const AutorunRootKey: TDWAutorunRootKey; const AutorunOption: TDWAutorunOption; const ValueName: String): Boolean;
function dwWriteRegColor(const Color: TColor; const RootKey: HKEY; const KeyName, ValueName: String): Boolean;

implementation

uses dwConsts, dwConvert, dwInternal, dwStrings,
     Classes, Messages, Registry, SysUtils;

function dwAutostartIsRegistered(const AutorunRootKey: TDWAutorunRootKey;
                               const AutorunOption:  TDWAutorunOption;
                               const ValueName:      String): Boolean; overload;
begin
   result:= dwiAutostartIsRegistered(AutorunRootKey, AutorunOption, ValueName, '', FALSE);
end;

function dwAutostartIsRegistered(const AutorunRootKey: TDWAutorunRootKey;
                               const AutorunOption:  TDWAutorunOption;
                               const ValueName,
                                     CommandString:  String): Boolean; overload;
begin
   result:= dwiAutostartIsRegistered(AutorunRootKey, AutorunOption, ValueName, CommandString, TRUE);
end;

function dwGetAutostart(const AutorunRootKey: TDWAutorunRootKey;
                      const AutorunOption:  TDWAutorunOption;
                      const ValueName:      String): String;
var
   sOpenKey: String;
begin
   with TRegistry.Create do begin
      case AutorunRootKey of
      dwarkHKCU: RootKey:= HKEY_CURRENT_USER;
      dwarkHKLM: RootKey:= HKEY_LOCAL_MACHINE;
      end;
      case AutorunOption of
      dwaoRun:             sOpenKey:= sBaseKey + sRun;
      dwaoRunOnce:         sOpenKey:= sBaseKey + sRunOnce;
      dwaoRunOnceEx:       sOpenKey:= sBaseKey + sRunOnceEx;
      dwaoRunServices:     sOpenKey:= sBaseKey + sRunServices;
      dwaoRunServicesOnce: sOpenKey:= sBaseKey + sRunServicesOnce;
      end;
      Free;
   end;
   result:= sOpenKey;
end;

function dwRefreshIconCache: Boolean;
const
  KEY_TYPE = HKEY_CURRENT_USER;
  KEY_NAME = 'Control Panel\Desktop\WindowMetrics';
  KEY_VALUE = 'Shell Icon Size';
var
  Reg: TRegistry;
  strDataRet, strDataRet2: string;
 procedure BroadcastChanges;
 var
   success: DWORD;
 begin
   SendMessageTimeout(HWND_BROADCAST,
                      WM_SETTINGCHANGE,
                      SPI_SETNONCLIENTMETRICS,
                      0,
                      SMTO_ABORTIFHUNG,
                      10000,
                      success);
 end;
begin
  Result := False;
  Reg := TRegistry.Create;
  try
    Reg.RootKey := KEY_TYPE;
    // 1. open HKEY_CURRENT_USER\Control Panel\Desktop\WindowMetrics
    if Reg.OpenKey(KEY_NAME, False) then
    begin
      // 2. Get the value for that key
      strDataRet := Reg.ReadString(KEY_VALUE);
      Reg.CloseKey;
      if strDataRet <> '' then
      begin
        // 3. Convert sDataRet to a number and subtract 1,
        //    convert back to a string, and write it to the registry
        strDataRet2 := IntToStr(StrToInt(strDataRet) - 1);
        if Reg.OpenKey(KEY_NAME, False) then
        begin
          Reg.WriteString(KEY_VALUE, strDataRet2);
          Reg.CloseKey;
          // 4. because the registry was changed, broadcast
          //    the fact passing SPI_SETNONCLIENTMETRICS,
          //    with a timeout of 10000 milliseconds (10 seconds)
          BroadcastChanges;
          // 5. the desktop will have refreshed with the
          //    new (shrunken) icon size. Now restore things
          //    back to the correct settings by again writing
          //    to the registry and posing another message.
          if Reg.OpenKey(KEY_NAME, False) then
          begin
            Reg.WriteString(KEY_VALUE, strDataRet);
            Reg.CloseKey;
            // 6.  broadcast the change again
            BroadcastChanges;
            Result := True;
          end;
        end;
      end;
    end;
  finally
    Reg.Free;
  end;
end;

function dwRegisterAutostart(const AutorunRootKey:    TDWAutorunRootKey;
                           const AutorunOption:     TDWAutorunOption;
                           const ValueName,
                                 CommandString:     String;
                           const OverwriteExisting: Boolean): Boolean;
{const
   sBaseKey =         '\Software\Microsoft\Windows\CurrentVersion\';
   sRun =             'Run\';
   sRunOnce =         'RunOnce\';
   sRunOnceEx =       'RunOnceEx\';
   sRunServices =     'RunServices\';
   sRunServicesOnce = 'RunServicesOnce\';}
var
   bCanOpenKey,
   bCanWriteValue,
   bValueExists: Boolean;
   sOpenKey: String;
begin
   bCanWriteValue:= FALSE;
   with TRegistry.Create do begin
      case AutorunRootKey of
      dwarkHKCU: RootKey:= HKEY_CURRENT_USER;
      dwarkHKLM: RootKey:= HKEY_LOCAL_MACHINE;
      end;
      case AutorunOption of
      dwaoRun:             sOpenKey:= sBaseKey + sRun;
      dwaoRunOnce:         sOpenKey:= sBaseKey + sRunOnce;
      dwaoRunOnceEx:       sOpenKey:= sBaseKey + sRunOnceEx;
      dwaoRunServices:     sOpenKey:= sBaseKey + sRunServices;
      dwaoRunServicesOnce: sOpenKey:= sBaseKey + sRunServicesOnce;
      end;
      bCanOpenKey:= OpenKey(sOpenKey, FALSE);
      if bCanOpenKey then begin
         bValueExists:= ValueExists(ValueName);
         bCanWriteValue:= ((bValueExists and OverwriteExisting) or (not bValueExists));
         if bCanWriteValue then WriteString(ValueName, CommandString);
      end;
      Free;
      result:= (bCanOpenKey and bCanWriteValue);
   end;
end;

function dwUnregisterAutostart(const AutorunRootKey:    TDWAutorunRootKey;
                             const AutorunOption:     TDWAutorunOption;
                             const ValueName:         String): Boolean;
{const
   sBaseKey =         '\Software\Microsoft\Windows\CurrentVersion\';
   sRun =             'Run\';
   sRunOnce =         'RunOnce\';
   sRunOnceEx =       'RunOnceEx\';
   sRunServices =     'RunServices\';
   sRunServicesOnce = 'RunServicesOnce\';}
var
   bCanOpenKey,
   bValueExists: Boolean;
   sOpenKey: String;
begin
   bValueExists:= FALSE;
   with TRegistry.Create do begin
      case AutorunRootKey of
      dwarkHKCU: RootKey:= HKEY_CURRENT_USER;
      dwarkHKLM: RootKey:= HKEY_LOCAL_MACHINE;
      end;
      case AutorunOption of
      dwaoRun:             sOpenKey:= sBaseKey + sRun;
      dwaoRunOnce:         sOpenKey:= sBaseKey + sRunOnce;
      dwaoRunOnceEx:       sOpenKey:= sBaseKey + sRunOnceEx;
      dwaoRunServices:     sOpenKey:= sBaseKey + sRunServices;
      dwaoRunServicesOnce: sOpenKey:= sBaseKey + sRunServicesOnce;
      end;
      bCanOpenKey:= OpenKey(sOpenKey, FALSE);
      if bCanOpenKey then begin
         bValueExists:= ValueExists(ValueName);
         if bValueExists then DeleteValue(ValueName);
      end;
      result:= (bCanOpenKey and bValueExists);
      Free;
   end;
end;

function dwReadRegColor(const RootKey: HKEY; const KeyName, ValueName: String): TColor;
var
   Reg: TRegistry;
   DataInfo: TRegDataInfo;
begin
   result:= clNone;
   Reg:= TRegistry.Create(KEY_READ);
   Reg.RootKey:= RootKey;
   with Reg do begin
      if OpenKey(KeyName, FALSE) then begin
         GetDataInfo(ValueName, DataInfo);
         if (DataInfo.RegData = rdString) then begin
            result:= StringToColor(dwRegColorToStr(ReadString(ValueName)));
         end;
      end;
      Free;
   end;
end;

function dwWriteRegColor(const Color: TColor; const RootKey: HKEY; const KeyName, ValueName: String): Boolean;
var
   Reg: TRegistry;
begin
   result:= FALSE;
   Reg:= TRegistry.Create;
   with Reg do begin
      RootKey:= RootKey;
      if OpenKey(KeyName, TRUE) then begin
         WriteString(ValueName, dwColorToRegColor(Color));
         result:= TRUE;
      end;
      Free;
   end;
end;

end.
 