{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwScreens.pas
 Description: Functions for graphic cards and monitors
 Author:      This unit was completely developed by Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     14.12.2002
 Last mod.:   16.08.2004
 History:     n/a
-----------------------------------------------------------------------------}
unit dwScreens;

interface

uses dwTypes,
     Classes, Graphics, MultiMon, Windows;

function dwCenterWindowToScreen(hWnd: HWND; const Index: Byte): Boolean;
function dwChangeScreenMode(const Index: Byte; const ModeIndex: Integer; const MonitorSupportedOnly, SavePermanent: Boolean): Boolean; overload;
function dwChangeScreenMode(const Index: Byte; var ScreenMode: TDWScreenMode): Boolean; overload;
function dwGetMonitorDevMode(const Index: Byte; var DevMode: TDWDevMode): Boolean;
function dwGetMonitorFixedResolutionMode(const Index: Byte): TDWFixedResolutionMode;
function dwGetMonitorHandle(const Index: Byte): HMonitor;
function dwGetMonitorInfoEx(const Index: Byte; var MonitorInfoEx: TDWMonitorInfoEx): Boolean;
function dwGetMonitorOrientation(const Index: Byte): TDWDisplayOrientation;
function dwGetMonitorRect(const Index: Byte): TRect;
function dwGetMonitorScreenMode(const Index: Byte; var ScreenMode: TDWScreenMode): Boolean;
function dwGetMonitorWorkRect(const Index: Byte): TRect;
function dwGetScreenModeByIndex(const Index: Byte; const ModeIndex: Integer; const MonitorSupportedOnly: Boolean; var ScreenMode: TDWScreenMode): Boolean;
function dwGetScreenModeList(const Index: Byte; const MonitorSupportedOnly: Boolean; var ModeList: TDWScreenModeList): Boolean;
function dwGetWindowMonitorHandle(hWnd: HWND): HMonitor;
function dwGetWindowScreenIndex(hWnd: HWND): Byte;
function dwMakeMonitorShapshot(const Index: Byte; Bitmap: Graphics.TBitmap): Boolean;
function dwMaximizeWindowToScreen(hWnd: HWND; const Index: Byte): Boolean;
function dwMonitorIsGrayscale(const Index: Byte): Boolean;
function dwMonitorIsInterlaced(const Index: Byte): Boolean;
function dwMoveWindowToScreen(hWnd: HWND; const Index: Byte): Boolean;
function dwScreenColorDepth(const Index: Byte): Byte;
function dwScreenCount(const ScreenType: TDWScreenType): Integer;
function dwScreenDeviceName(const Index: Byte): String;
function dwScreenDeviceString(const Index: Byte): String;
function dwScreenDeviceType(const Index: Byte): TDWScreenTypes;
function dwScreenMonitorFrequency(const Index: Byte): Integer;
function dwScreenMonitorName(const Index: Byte): String;
function dwScreenMonitorPosition(const Index: Byte; var Point: TPoint): Boolean;
function dwScreenMonitorString(const Index: Byte): String;
function dwScreenResolution(const Index: Byte): TDWDimension;
function dwWindowFitToMonitorWorkArea(hWnd: HWND; Index: Byte): Boolean;

procedure dwMonitorOff;
procedure dwMonitorOn;
procedure dwScreenDeviceList(const ScreenType: TDWScreenType; List: TStrings);
procedure dwScreenMonitorList(const ScreenType: TDWScreenType; List: TStrings);

{$DEFINE WINVER_0400_GREATER}

implementation

uses dwConsts, dwInternal, dwWinSystem,
	  Messages;

function dwCenterWindowToScreen(hWnd: HWND; const Index: Byte): Boolean;
var
	WindowRect: TRect;
   MonitorCoord: TPoint;
   MonitorResolution: TDWDimension;
   Success: Boolean;
   WindowHeight, WindowWidth, WindowX, WindowY: Integer;
begin
   result:= FALSE;
   Success:= TRUE;
   try
      if not (dwscrtDesktop in dwScreenDeviceType(Index)) then exit;
      Success:= Success and GetWindowRect(hWnd, WindowRect);
      Success:= Success and dwScreenMonitorPosition(Index, MonitorCoord);
      MonitorResolution:= dwScreenResolution(Index);
      WindowHeight:= WindowRect.Bottom - WindowRect.Top;
      WindowWidth:= WindowRect.Right - WindowRect.Left;
      if (WindowHeight > MonitorResolution.Height) or
         (WindowWidth > MonitorResolution.Width) then exit;
      WindowX:= ((MonitorResolution.Width  - WindowWidth) div 2) + MonitorCoord.x;
      WindowY:= ((MonitorResolution.Height - WindowHeight) div 2) + MonitorCoord.y;
      Success:= Success and MoveWindow(hWnd, WindowX, WindowY, WindowWidth,
                                       WindowHeight, TRUE);
   except
   	exit;
   end;
   result:= Success;
end;

function dwMonitorIsGrayscale(const Index: Byte): Boolean;
var
	DevMode: TDWDevMode;
begin
{$IFDEF WINVER_0400_GREATER}
   ZeroMemory(@DevMode, SizeOf(TDWDevMode));
   DevMode.dmSize:= SizeOf(TDWDevMode);
   EnumDisplaySettingsExA(PChar(dwScreenDeviceName(Index)),
                          ENUM_CURRENT_SETTINGS,
                          DevMode,
                          0);
   result:= ((DevMode.dmDisplayFlagsUnion.dmDisplayFlags and DM_GRAYSCALE) =
              DM_GRAYSCALE);
{$ELSE}
   result:= FALSE;
{$ENDIF}
end;

function dwMonitorIsInterlaced(const Index: Byte): Boolean;
var
	DevMode: TDWDevMode;
begin
{$IFDEF WINVER_0400_GREATER}
   ZeroMemory(@DevMode, SizeOf(TDWDevMode));
   DevMode.dmSize:= SizeOf(TDWDevMode);
   EnumDisplaySettingsExA(PChar(dwScreenDeviceName(Index)),
                          ENUM_CURRENT_SETTINGS,
                          DevMode,
                          0);
   result:= ((DevMode.dmDisplayFlagsUnion.dmDisplayFlags and DM_INTERLACED) =
              DM_INTERLACED);
{$ELSE}
	result:= FALSE;
{$ENDIF}
end;

function dwChangeScreenMode(const Index: Byte; var ScreenMode: TDWScreenMode): Boolean; overload;
var
	DevMode: TDWDevMode;
begin
   result:= TRUE;
	result:= result and dwGetMonitorDevMode(Index, DevMode);
   dwiMergeScreenModeToDevMode(DevMode, ScreenMode);
	result:= result and
            (dwInternal.ChangeDisplaySettingsEx(PChar(dwScreenDeviceName(Index)),
                                                @DevMode,
                                                0,
                                                0,
                                                NIL) = DISP_CHANGE_SUCCESSFUL);
end;

function dwChangeScreenMode(const Index: Byte; const ModeIndex: Integer; const MonitorSupportedOnly, SavePermanent: Boolean): Boolean; overload;
var
	DevMode: TDWDevMode;
   ScreenMode: TDWScreenMode;
   Flags: DWord;
begin
{$IFDEF WINVER_0400_GREATER}
   result:= TRUE;
   if MonitorSupportedOnly then Flags:= 0 else Flags:= EDS_RAWMODE;
   ZeroMemory(@DevMode, SizeOf(TDWDevMode));
   DevMode.dmSize:= SizeOf(TDWDevMode);
   result:= result and EnumDisplaySettingsExA(PChar(dwScreenDeviceName(Index)),
                                              ENUM_CURRENT_SETTINGS,
                                              DevMode,
                                              Flags);

   result:= result and dwGetScreenModeByIndex(Index,
   														 ModeIndex,
                                              MonitorSupportedOnly,
                                              ScreenMode);

   dwiMergeScreenModeToDevMode(DevMode, ScreenMode);

	result:= result and
            (dwInternal.ChangeDisplaySettingsEx(PChar(dwScreenDeviceName(Index)),
                                                @DevMode,
                                                0,
                                                0,
                                                NIL) = DISP_CHANGE_SUCCESSFUL);
{$ELSE}
	result:= FALSE;
{$ENDIF}
end;

function dwGetMonitorDevMode(const Index: Byte; var DevMode: TDWDevMode): Boolean;
begin
{$IFDEF WINVER_0400_GREATER}
   result:= TRUE;
   ZeroMemory(@DevMode, SizeOf(TDWDevMode));
   DevMode.dmSize:= SizeOf(TDWDevMode);
   result:= result and EnumDisplaySettingsExA(PChar(dwScreenDeviceName(Index)),
                                              ENUM_CURRENT_SETTINGS,
                                              DevMode,
                                              0);
{$ELSE}
	result:= FALSE;
{$ENDIF}
end;

function dwGetMonitorFixedResolutionMode(const Index: Byte): TDWFixedResolutionMode;
var
	DevMode: TDWDevMode;
begin
{$IFDEF WINVER_0400_GREATER}
   ZeroMemory(@DevMode, SizeOf(TDWDevMode));
   DevMode.dmSize:= SizeOf(TDWDevMode);
   DevMode.dmFields:= DM_DISPLAYFIXEDOUTPUT;
   EnumDisplaySettingsExA(PChar(dwScreenDeviceName(Index)),
                          ENUM_CURRENT_SETTINGS,
                          DevMode,
                          0);
   case DevMode.dmPaperUnion.dmDisplayFixedOutput of
   	DMDFO_DEFAULT: 	result:= dwfrmDefault;
      DMDFO_STRETCH:    result:= dwfrmStretch;
      DMDFO_CENTER:		result:= dwfrmStretch;
      else					result:= dwfrmDefault;
   end;
{$ELSE}
	result:= dwfrmDefault;
{$ENDIF}
end;

function dwGetMonitorHandle(const Index: Byte): HMonitor;
var
	pt: TPoint;
begin
   ZeroMemory(@result, SizeOf(HMonitor));
   dwScreenMonitorPosition(Index, pt);
	with pt do begin
   	x:= x + 1;
      y:= y + 1;
   end;
   result:= MonitorFromPoint(pt, MONITOR_DEFAULTTONEAREST);
end;

function dwGetMonitorOrientation(const Index: Byte): TDWDisplayOrientation;
var
	Dim: TDWDimension;
	DevMode: TDWDevMode;
begin
{$IFDEF WINVER_0400_GREATER}
   if not dwIsWinXP then begin
      Dim:= dwScreenResolution(Index);
      with Dim do begin
         if Height > Width then result:= dwdoPortrait else result:= dwdoLandscape;
      end;
   end else begin
      ZeroMemory(@DevMode, SizeOf(TDWDevMode));
      DevMode.dmSize:= SizeOf(TDWDevMode);
      DevMode.dmFields:= DM_ORIENTATION;
      EnumDisplaySettingsExA(PChar(dwScreenDeviceName(Index)),
                             ENUM_CURRENT_SETTINGS,
                             DevMode,
                             0);
      case DevMode.dmPaperUnion.dmDisplayOrientation of
      DMDO_DEFAULT: result:= dwdoDefault;
      DMDO_90:		  result:= dwdo90Degrees;
      DMDO_180:	  result:= dwdo180Degrees;
      DMDO_270:	  result:= dwdo270Degrees;
      else			  result:= dwdoDefault;
      end;
   end;
{$ELSE}
	result:= dwdoDefault;
{$ENDIF}
end;

function dwGetMonitorInfoEx(const Index: Byte; var MonitorInfoEx: TDWMonitorInfoEx): Boolean;
begin
   ZeroMemory(@MonitorInfoEx, SizeOf(TDWMonitorInfoEx));
	MonitorInfoEx.cbSize:= SizeOf(TDWMonitorInfoEx);
   result:= GetMonitorInfo(dwGetMonitorHandle(Index), @MonitorInfoEx);
end;

function dwGetMonitorRect(const Index: Byte): TRect;
var
	MIE: TDWMonitorInfoEx;
begin
	dwGetMonitorInfoEx(Index, MIE);
   result:= MIE.rcMonitor;
end;

function dwGetMonitorScreenMode(const Index: Byte; var ScreenMode: TDWScreenMode): Boolean;
var
	DevMode: TDWDevMode;
begin
	result:= TRUE;
   result:= result and dwGetMonitorDevMode(Index, DevMode);
   dwiScreenModeFromDevMode(ScreenMode, DevMode);
end;

function dwGetMonitorWorkRect(const Index: Byte): TRect;
var
	MIE: TDWMonitorInfoEx;
begin
	dwGetMonitorInfoEx(Index, MIE);
   result:= MIE.rcWork;
end;

function dwGetScreenModeByIndex(const Index: Byte; const ModeIndex: Integer; const MonitorSupportedOnly: Boolean; var ScreenMode: TDWScreenMode): Boolean;
var
	I: Integer;
   DevMode: TDWDevMode;
   Flags: DWord;
begin
{$IFDEF WINVER_0400_GREATER}
	I:= 0;
   ZeroMemory(@DevMode, SizeOf(TDWDevMode));
   DevMode.dmSize:= SizeOf(TDWDevMode);
   if MonitorSupportedOnly then Flags:= 0 else Flags:= EDS_RAWMODE;
   while EnumDisplaySettingsExA(PChar(dwScreenDeviceName(Index)),
                                I,
                                DevMode,
                                Flags) do
   begin
      if ModeIndex = I then dwiScreenModeFromDevMode(ScreenMode, DevMode)
      else Inc(I);
   end;
   result:= (I > 0);
{$ELSE}
	result:= FALSE;
{$ENDIF}
end;

function dwGetScreenModeList(const Index: Byte; const MonitorSupportedOnly: Boolean; var ModeList: TDWScreenModeList): Boolean;
var
	I: Integer;
   DevMode: TDWDevMode;
   Flags: DWord;
begin
{$IFDEF WINVER_0400_GREATER}
	I:= 0;
   SetLength(ModeList, I);
   ZeroMemory(@DevMode, SizeOf(TDWDevMode));
   DevMode.dmSize:= SizeOf(TDWDevMode);
   if MonitorSupportedOnly then Flags:= 0 else Flags:= EDS_RAWMODE;
   while EnumDisplaySettingsExA(PChar(dwScreenDeviceName(Index)),
                                I,
                                DevMode,
                                Flags) do
   begin
   	SetLength(ModeList, I + 1);
		dwiScreenModeFromDevMode(ModeList[I], DevMode);
      Inc(I);
   end;
   result:= (I > 0);
{$ELSE}
	result:= FALSE;
{$ENDIF}
end;

function dwGetWindowMonitorHandle(hWnd: HWND): HMonitor;
begin
	result:= MonitorFromWindow(hWnd, MONITOR_DEFAULTTONEAREST);
end;

function dwGetWindowScreenIndex(hWnd: HWND): Byte;
var
	MIE: TDWMonitorInfoEx;
   Handle: HMonitor;
   I: Integer;
begin
   result:= 0;
   Handle:= dwGetWindowMonitorHandle(hWnd);
   ZeroMemory(@MIE, SizeOf(TDWMonitorInfoEx));
   MIE.cbSize:= SizeOf(TDWMonitorInfoEx);
   GetMonitorInfo(Handle, @MIE);
   for I:= 0 to dwScreenCount(dwscrtAll) - 1 do begin
   	if MIE.szService = dwScreenDeviceName(I) then begin
      	result:= I;
         Break;
      end;
   end;
end;

function dwMakeMonitorShapshot(const Index: Byte; Bitmap: Graphics.TBitmap): Boolean;
var
  	Desktop: HDC;
  	Left, Top, Height, Width: Integer;
  	rcMonitor: TRect;
begin
   rcMonitor:= dwGetMonitorRect(Index);
   Left:= rcMonitor.Left;
   Top:= rcMonitor.Top;
   Height:= rcMonitor.Bottom - rcMonitor.Top;
   Width:= rcMonitor.Right - rcMonitor.Left;
  	Bitmap.Height := Height;
  	Bitmap.Width := Width;
  	Desktop := CreateDC('', PChar(dwScreenDeviceName(Index)), NIL, NIL);
  	BitBlt(Bitmap.Canvas.Handle, 0, 0, Width, Height, Desktop, 0, 0, SRCCOPY);
  	DeleteDC(Desktop);
end;

function dwMaximizeWindowToScreen(hWnd: HWND; const Index: Byte): Boolean;
var
	WindowRect: TRect;
   MonitorCoord: TPoint;
   MonitorResolution: TDWDimension;
   Success: Boolean;
   WindowHeight, WindowWidth, WindowX, WindowY: Integer;
begin
   result:= FALSE;
   if dwGetWindowScreenIndex(hWnd) = Index then exit;
   Success:= TRUE;
   try
      if not (dwscrtDesktop in dwScreenDeviceType(Index)) then exit;
      Success:= Success and GetWindowRect(hWnd, WindowRect);
      Success:= Success and dwScreenMonitorPosition(Index, MonitorCoord);
      MonitorResolution:= dwScreenResolution(Index);
      WindowHeight:= WindowRect.Bottom - WindowRect.Top;
      WindowWidth:= WindowRect.Right - WindowRect.Left;
//      if (WindowHeight > MonitorResolution.Height) or
//         (WindowWidth > MonitorResolution.Width) then exit;
      WindowX:= ((MonitorResolution.Width  - WindowWidth) div 2) + MonitorCoord.x;
      WindowY:= ((MonitorResolution.Height - WindowHeight) div 2) + MonitorCoord.y;
      Success:= Success and ShowWindow(hWnd, SW_HIDE);
      Success:= Success and MoveWindow(hWnd, WindowX, WindowY, WindowWidth,
                                       WindowHeight, TRUE);
      Success:= Success and ShowWindow(hWnd, SW_MAXIMIZE);
      Success:= Success and ShowWindow(hWnd, SW_SHOW);
   except
   	exit;
   end;
   result:= Success;
end;

function dwMoveWindowToScreen(hWnd: HWND; const Index: Byte): Boolean;
var
	CurrentMonitorRect, DestMonitorRect, WindowRect: TRect;
   RelativeX, RelativeY: Integer;
begin
   result:= TRUE;
	CurrentMonitorRect:= dwGetMonitorRect(dwGetWindowScreenIndex(hWnd));
   DestMonitorRect:= dwGetMonitorRect(Index);
   result:= result and GetWindowRect(hWnd, WindowRect);
   RelativeX:= WindowRect.Left - CurrentMonitorRect.Left;
   RelativeY:= WindowRect.Top - CurrentMonitorRect.Top;
   result:= result and MoveWindow(hWnd,
                                  DestMonitorRect.Left + RelativeX,
                                  DestMonitorRect.Top + RelativeY,
                                  WindowRect.Right - WindowRect.Left,
                                  WindowRect.Bottom - WindowRect.Top,
                                  TRUE);
end;

function dwScreenColorDepth(const Index: Byte): Byte;
var
   DevMode: TDeviceMode;
begin
   result:= 0;
   DevMode.dmSize:= SizeOf(DevMode);
   if EnumDisplaySettings(PChar(dwScreenDeviceName(Index)), ENUM_CURRENT_SETTINGS,
                          DevMode) then begin
      result:= DevMode.dmBitsPerPel;
   end;
end;

function dwScreenCount(const ScreenType: TDWScreenType): Integer;
var
   DeviceCount: Integer;
   DisplayDevice: TDisplayDevice;
   DeviceNum: DWord;
begin
   result:= 1;
   if (dwIsNT4 or dwIsWin95) then exit; //These OS does not support multimon
//   result:= 0;
   DisplayDevice.cb:= SizeOf(DisplayDevice);
   DeviceNum:= 0;
   DeviceCount:= 1;
   while EnumDisplayDevices(NIL, DeviceNum, DisplayDevice, 0) do begin
      case ScreenType of
      dwscrtAll:
         begin
            if (DisplayDevice.StateFlags and DISPLAY_DEVICE_PRIMARY_DEVICE)>0 then
               Dec(DeviceCount);
            Inc(DeviceCount);
         end;
      dwscrtDesktop:
         begin
            if (DisplayDevice.StateFlags and DISPLAY_DEVICE_PRIMARY_DEVICE)>0 then
               Dec(DeviceCount);
            if (DisplayDevice.StateFlags and DISPLAY_DEVICE_ATTACHED_TO_DESKTOP)>0 then
               Inc(DeviceCount);
         end;
      dwscrtMirror:
         begin
            if (DisplayDevice.StateFlags and DISPLAY_DEVICE_PRIMARY_DEVICE)>0 then
               Dec(DeviceCount);
            if (DisplayDevice.StateFlags and DISPLAY_DEVICE_MIRRORING_DRIVER)>0 then
               Inc(DeviceCount);
         end;
      dwscrtRemovable:
         begin
            if (DisplayDevice.StateFlags and DISPLAY_DEVICE_PRIMARY_DEVICE)>0 then
               Dec(DeviceCount);
            if (DisplayDevice.StateFlags and DISPLAY_DEVICE_REMOVABLE)>0 then
               Inc(DeviceCount);
         end;
      dwscrtVGA:
         begin
            if (DisplayDevice.StateFlags and DISPLAY_DEVICE_PRIMARY_DEVICE)>0 then
               Dec(DeviceCount);
            if (DisplayDevice.StateFlags and DISPLAY_DEVICE_VGA_COMPATIBLE)>0 then
               Inc(DeviceCount);
         end;
      end;
      Inc(DeviceNum);
   end;
   result:= DeviceCount;
end;

function dwScreenDeviceName(const Index: Byte): String;
var
   DisplayDevice: TDisplayDevice;
begin
   result:= '';
   DisplayDevice.cb:= SizeOf(DisplayDevice);
   if EnumDisplayDevices(NIL, Index, DisplayDevice, 0) then begin
      result:= DisplayDevice.DeviceName;
   end;
end;

function dwScreenDeviceString(const Index: Byte): String;
var
   DisplayDevice: TDisplayDevice;
begin
   result:= '';
   DisplayDevice.cb:= SizeOf(DisplayDevice);
   if EnumDisplayDevices(NIL, Index, DisplayDevice, 0) then begin
      result:= DisplayDevice.DeviceString;
   end;
end;

function dwScreenDeviceType(const Index: Byte): TDWScreenTypes;
var
   DisplayDevice: TDisplayDevice;
begin
   result:= [];
   DisplayDevice.cb:= SizeOf(DisplayDevice);
   if EnumDisplayDevices(NIL, Index, DisplayDevice, 0) then begin
      with DisplayDevice do begin
         if ((StateFlags and DISPLAY_DEVICE_ATTACHED_TO_DESKTOP) =
            DISPLAY_DEVICE_ATTACHED_TO_DESKTOP) then Include(result, dwscrtDesktop);

         if ((StateFlags and DISPLAY_DEVICE_MIRRORING_DRIVER) =
            DISPLAY_DEVICE_MIRRORING_DRIVER) then Include(result, dwscrtMirror);

         if ((StateFlags and DISPLAY_DEVICE_MODESPRUNED) =
            DISPLAY_DEVICE_MODESPRUNED) then Include(result, dwscrtModeSpruned);

         if ((StateFlags and DISPLAY_DEVICE_PRIMARY_DEVICE) =
            DISPLAY_DEVICE_PRIMARY_DEVICE) then Include(result, dwscrtPrimary);

         if ((StateFlags and DISPLAY_DEVICE_REMOVABLE) =
            DISPLAY_DEVICE_REMOVABLE) then Include(result, dwscrtRemovable);

         if ((StateFlags and DISPLAY_DEVICE_VGA_COMPATIBLE) =
            DISPLAY_DEVICE_VGA_COMPATIBLE) then Include(result, dwscrtVGA);
      end;
   end;
end;

function dwScreenMonitorFrequency(const Index: Byte): Integer;
var
   DevMode: TDeviceMode;
begin
   result:= 0;
   DevMode.dmSize:= SizeOf(TDeviceMode);
   if EnumDisplaySettings(PChar(dwScreenDeviceName(Index)), ENUM_CURRENT_SETTINGS,
                          DevMode) then begin
      result:= DevMode.dmDisplayFrequency;
   end;
end;

function dwScreenMonitorName(const Index: Byte): String;
var
   DisDev, DisDevMon: TDisplayDevice;
begin
   result:= '';
   DisDev.cb:= SizeOf(TDisplayDevice);
   DisDevMon.cb:= SizeOf(TDisplayDevice);
   if EnumDisplayDevices(NIL, Index, DisDev, 0) then begin
      if DisDev.StateFlags = DISPLAY_DEVICE_MIRRORING_DRIVER then exit;
      while EnumDisplayDevices(@DisDev.DeviceName, 0, DisDevMon, 0) do begin
         result:= DisDevMon.DeviceName;
         if (DisDevMon.StateFlags <> DISPLAY_DEVICE_ACTIVE) then break;
      end;
   end;
end;

function dwScreenMonitorPosition(const Index: Byte; var Point: TPoint): Boolean;
var
  DisDev, DisDevMon: TDisplayDevice;
  realDevMode: TDWDevMode;
  bCanEnumExMonitor:Boolean;
begin
	result:= FALSE;
   with Point do
   begin
     x := 0;
     y := 0;
   end;
   ZeroMemory(@DisDev, SizeOf(TDisplayDevice));
   DisDev.cb := SizeOf(TDisplayDevice);
   if not EnumDisplayDevices(nil, Index, DisDev, 0) then exit;
   DisDevMon.cb := SizeOf(TDisplayDevice);
   if not EnumDisplayDevices(@DisDev.DeviceName, 0, DisDevMon, 0) then exit;
   ZeroMemory(@realDevMode, SizeOf(TDWDevMode));
   realDevMode.dmSize := SizeOf(TDWDevMode);
   bCanEnumExMonitor := EnumDisplaySettingsExA(@DisDev.DeviceName[0], ENUM_CURRENT_SETTINGS, realDevMode, 0);
   if bCanEnumExMonitor then
   begin
     if (DisDev.StateFlags and DISPLAY_DEVICE_ATTACHED_TO_DESKTOP) =
         DISPLAY_DEVICE_ATTACHED_TO_DESKTOP then
     begin
       Point.x := realDevMode.dmPaperUnion.dmPosition.x;
       Point.y := realDevMode.dmPaperUnion.dmPosition.y;
     end;
   end;
   result:= bCanEnumExMonitor;
end;

function dwScreenMonitorString(const Index: Byte): String;
var
   DisDev, DisDevMon: TDisplayDevice;
begin
   result:= '';
   DisDev.cb:= SizeOf(TDisplayDevice);
   DisDevMon.cb:= SizeOf(TDisplayDevice);
   if EnumDisplayDevices(NIL, Index, DisDev, 0) then begin
      if DisDev.StateFlags = DISPLAY_DEVICE_MIRRORING_DRIVER then exit;
      while EnumDisplayDevices(@DisDev.DeviceName, 0, DisDevMon, 0) do begin
         result:= DisDevMon.DeviceString;
         if (DisDevMon.StateFlags <> DISPLAY_DEVICE_ACTIVE) then break;
      end;
   end;
end;

function dwScreenResolution(const Index: Byte): TDWDimension;
var
   DevMode: TDeviceMode;
begin
   with result do begin
      Height:= 0;
      Width:= 0;
   end;
   if EnumDisplaySettings(PChar(dwScreenDeviceName(Index)), ENUM_CURRENT_SETTINGS,
                          DevMode) then begin
      with result do begin
         Height:= DevMode.dmPelsHeight;
         Width:= DevMode.dmPelsWidth;
      end;
   end;
end;

function dwWindowFitToMonitorWorkArea(hWnd: HWND; Index: Byte): Boolean;
var
	MWA: TRect;
begin
	MWA:= dwGetMonitorWorkRect(Index);
   result:= MoveWindow(hWnd,
   						    MWA.Left,
                         MWA.Top,
                         MWA.Right - MWA.Left - 1,
                         MWA.Bottom - MWA.Top - 1,
                         TRUE);
end;

{-----------------------------------------------------------------------------
  Procedures
-----------------------------------------------------------------------------}
procedure dwMonitorOff;
begin
   SendMessage(GetCurrentProcess, WM_SYSCOMMAND, SC_MONITORPOWER, 0);
end;

procedure dwMonitorOn;
begin
   SendMessage(GetCurrentProcess, WM_SYSCOMMAND, SC_MONITORPOWER, -1);
end;

procedure dwScreenDeviceList(const ScreenType: TDWScreenType; List: TStrings);
var
   I: Integer;
begin
   for I:= 0 to dwScreenCount(dwscrtAll) - 1 do begin
      if ScreenType <> dwscrtAll then begin
        if (ScreenType in dwScreenDeviceType(I))  then
           List.Add(dwScreenDeviceString(I));
      end else begin
        List.Add(dwScreenDeviceString(I));
      end;
   end;
end;

procedure dwScreenMonitorList(const ScreenType: TDWScreenType; List: TStrings);
var
   I: Integer;
begin
   for I:= 0 to dwScreenCount(dwscrtAll) - 1 do begin
      if ScreenType <> dwscrtAll then begin
        if (ScreenType in dwScreenDeviceType(I))  then
           List.Add(dwScreenMonitorString(I));
      end else begin
        List.Add(dwScreenMonitorString(I));
      end;
   end;
end;

end.
