{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwStrings.pas
 Description: String operations
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     15.12.2002
 Last mod.:   16.08.2004
 History:     n/a
-----------------------------------------------------------------------------}

unit dwStrings;

interface

uses Classes;

function dwCodeString(Strng, Code : String) : String;
function dwCodeStringDelim(Str, Password: String; Delimiter: Char): String;
function dwCountCharInStr(Str: String; Chr: Char): Integer;
function dwDecodeStringDelim(Str: String; Delimiter: Char): String;
function dwHighPos(const Str, SubStr: String): Integer;
function dwReplaceStr(OldStr, FillStr: String; ReplaceChar: Char; AlignRight: Boolean): String;
function dwReverseStr(const Str: String): String;
function dwSmashStr(ValueA, ValueB: String): String;
function dwStrClearToInteger(Str: String): String;
function dwStringCountInStr(SubStr, Str: String): Integer;
function dwStrIsInteger(Str: String): Boolean;
function dwStrLeft(Str: String; Count: Integer): String;
function dwStrMid(Str: String; Start, Count: Integer): String;
function dwStrPart(Str: String; Start, Stop: Integer): String;
function dwStrRight(Str: String; Count: Integer): String;
function dwSubPositionByIndex(Str, SubStr: String; Index: Integer): Integer;

procedure dwReverseStringList(var List: TStringList);

implementation

uses dwConvert,
     SysUtils;

function dwCodeString(Strng, Code : String) : String;
var
   i : byte;
   h : string;
   j : byte;
begin
   Code:= dwReverseStr(Code);
   i := 1;
   h := '';
   j := 1;
   while (i <= Length(strng)) do
   begin
      h := h + Chr(Ord(strng[i]) xor Ord(code[j]));
      Inc(i);
      if j = Length(code) then j := 1 else Inc(j);
   end;
   result:= h;
end;

function dwCodeStringDelim(Str, Password: String; Delimiter: Char): String;
begin
   Password:= dwReverseStr(Password);
   result:= dwCodeString(Str, Password) + Delimiter + Password;
end;

function dwCountCharInStr(Str: String; Chr: Char): Integer;
var
   I: Integer;
begin
   result:= 0;
   for I:= 1 to Length(Str) do begin
      if Str[I] = Chr then Inc(result);
   end;
end;

function dwDecodeStringDelim(Str: String; Delimiter: Char): String;
var
   TL: TStringList;
   CodedStr, Pwd: String;
begin
   TL:= TStringList.Create;
   dwStrToList(Str, Delimiter, TL);
   CodedStr:= TL[0];
   Pwd:= TL[1];
   TL.Free;
   result:= dwCodeString(CodedStr, Pwd);
end;

function dwHighPos(const Str, SubStr: String): Integer;
begin
   result:= dwSubPositionByIndex(Str, SubStr, dwStringCountInStr(SubStr, Str));
end;

function dwReplaceStr(OldStr, FillStr: String; ReplaceChar: Char; AlignRight: Boolean): String;
var
   fcCount, I, J, K: Integer;
begin
   if (Length(OldStr) < 1) then exit;
   result:= OldStr;
   if (Length(FillStr) < 1) then exit;
   fcCount:= dwStringCountInStr(ReplaceChar, OldStr);
   if fcCount < 1 then exit;
   if not AlignRight then begin
      K:= 0;
      for I:= 1 to Length(OldStr) do begin
         if OldStr[I] = ReplaceChar then begin
            inc(K);
            if not (K > Length(FillStr)) then OldStr[I]:= FillStr[K];
         end;
      end;
   end else begin
      FillStr:= dwReverseStr(FillStr);
      K:= 0;
      for J:= Length(OldStr) downto 1 do begin
         if OldStr[J] = ReplaceChar then begin
            inc(K);
            if not (K > Length(FillStr)) then OldStr[J]:= FillStr[K];
         end;
      end;
   end;
   result:= OldStr;
end;

function dwReverseStr(const Str: String): String;
var
   I: Integer;
begin
   result:= Str;
   if Length(Str) < 2 then exit;
   for I:= Length(Str) downto 1 do begin
      result[I]:= Str[(Length(Str) - I) + 1];
   end;
end;

procedure dwReverseStringList(var List: TStringList);
var
   TempList: TStringList;
   I: Integer;
begin
   TempList:= TStringList.Create;
   for I:= List.Count - 1 downto 0 do begin
      TempList.Add(List[I]);
   end;
   List.Assign(TempList);
   TempList.Free;
end;

function dwSmashStr(ValueA, ValueB: String): String;
var
  i : Integer;
  strTemp : String;
begin
  strTemp := ValueB;
  for i := 1 to Length(ValueB) do
    if Pos(ValueB[i], ValueA) = 0 then
      strTemp := StringReplace(strTemp, ValueB[i], '', [rfReplaceAll]);
  Result := strTemp;
end;

function dwStrClearToInteger(Str: String): String;
var i: integer;
begin
   Result := Str;
   for i := Length(Result) downto 1 do
      if not (Result[i] in ['0'..'9']) then
         Delete(Result, i, 1);
end;

function dwStringCountInStr(SubStr, Str: String): Integer;
var
   StrLn, SubLn, I: Integer;
   TempStr: String;
begin
   StrLn:= Length(PChar(Str));
   SubLn:= Length(PChar(SubStr));
   TempStr:= Str;
   result:= 0;
   for I:= 0 to StrLn - 1 do begin
      TempStr:= Copy(Str, I + 1, SubLn);
      if TempStr = SubStr then Inc(result);
   end;
end;

function dwStrIsInteger(Str: String): Boolean;
begin
   result:= not (StrToIntDef(Str, -2147483647)=-2147483647);
end;

function dwStrLeft(Str: String; Count: Integer): String;
begin
   result:= Copy(Str, 1, Count);
end;

function dwStrMid(Str: String; Start, Count: Integer): String;
begin
   result:= Copy(Str, Start, Count);
end;

function dwStrPart(Str: String; Start, Stop: Integer): String;
begin
   result:= Copy(Str, Start, Stop - Start);
end;

function dwStrRight(Str: String; Count: Integer): String;
var StrLn: Integer;
begin
   StrLn:= StrLen(PChar(Str));
   result:= Copy(Str, StrLn - (Count - 1), Count);
end;

function dwSubPositionByIndex(Str, SubStr: String; Index: Integer): Integer;
var
   StrLn, SubLn, Lock, I: Integer;
   TempStr: String;
begin
   StrLn:= Length(PChar(Str));
   SubLn:= Length(PChar(SubStr));
   result:= 0;
   Lock:= 0;
   for I:= 0 to StrLn - 1 do begin
      TempStr:= Copy(Str, I + 1, SubLn);
      if TempStr = SubStr then Inc(Lock);
      if Lock = Index then begin
         result:= I + 1;
         break;
      end;
   end;
end;

end.
