{-----------------------------------------------------------------------------
                          ---> Project DelphiWorks <---
                                   Version 1.3
 -----------------------------------------------------------------------------
 Unit:        dwWinSystem.pas
 Description: Replacement for old DW's OSWorks.pas
 Author:      Codehunter Works
 Release:     1.3
 State:       Stable
 Date:        16.08.2004
 Created:     14.12.2002
 Last mod.:   16.08.2004
 History:     n/a
-----------------------------------------------------------------------------}
unit dwWinSystem;

interface

uses dwTypes,
     Windows;

function dwActiveDesktopEnabled: Boolean;
function dwChangeWallpaper(const ImageFile: String; const WallpaperOpt: TDWWallpaperOpt): Boolean;
function dwGetBootOption: TDWBootOption;
function dwGetIEVersion: TDWVersionBlockEx;
function dwGetIEVersionLS: TDWVersionBlock;
function dwGetIEVersionMS: TDWVersionBlock;
function dwGetMenuAlignment: TDWMenuAlignment;
function dwGetMinimizedWindowArrangement: TDWMinimizedWindowArrangement;
function dwGetNTServicePackName: String;
function dwGetOS: TDWOSType;
function dwGetOSName: String;
function dwGetProductType: TOSProductType;
function dwGetSuiteType: TOSSuiteTypes;
function dwGetWindowsLanguageStr: String;
function dwIsAdmin: Boolean;
function dwIsDomainController: Boolean;
function dwIsNT351: Boolean;
function dwIsNT4: Boolean;
function dwIsNTMachine: Boolean;
function dwIsNTServer: Boolean;
function dwIsNTWorkstation: Boolean;
function dwIsWin2K: Boolean;
function dwIsWin95: Boolean;
function dwIsWin98: Boolean;
function dwIsWinME: Boolean;
function dwIsWinXP: Boolean;
function dwLockWorkstation: Boolean;
function dwNTServicePackMajorVersion: Integer;
function dwNTServicePackMinorVersion: Integer;
function dwSetEnvironmentVariable(const VarName, VarValue: String; const SetAllUsers: Boolean): Boolean;
function dwVirtualMachineIsPresent: Boolean;
function dwVMWareIsPresent: Boolean;
function dwWinBuildNumber: Integer;
function dwWinMajorVersion: Integer;
function dwWinMinorVersion: Integer;
function dwWow64IsRunning(AppHandle: THandle): Boolean;

procedure dwGetWindowsVersionEx(var OSVIE: TOSVersionInfoEx);
procedure dwMinimizeAllWindows;
procedure dwShowDesktop;
procedure dwXPUserLogOut;

implementation

uses dwConsts, dwConvert, dwFiles, dwInternal, dwLanguage,
     Classes, Messages, Registry, SysUtils;

function dwActiveDesktopEnabled: Boolean;
var
  h: HWND;
begin
  h := FindWindow('Progman', nil);
  h := FindWindowEx(h, 0, 'SHELLDLL_DefView', nil);
  h := FindWindowEx(h, 0, 'Internet Explorer_Server', nil);
  Result := h <> 0;
end;

function dwChangeWallpaper(const ImageFile: String; const WallpaperOpt: TDWWallpaperOpt): Boolean;
begin
  	if not dwActiveDesktopEnabled then
   	result:= dwiChangeWallpaperGeneric(ImageFile, WallpaperOpt)
   else
   	result:= dwiChangeWallpaperActiveDesktop(ImageFile, WallpaperOpt);
end;

function dwGetBootOption: TDWBootOption;
begin
   case GetSystemMetrics(SM_CLEANBOOT) of
   1: result:= dwboFailSafe;
   2: result:= dwboFailSaveNet;
   else result:= dwboNormal;
   end;
end;

function dwGetIEVersion: TDWVersionBlockEx;
var
   FI: TDWFileInfo;
	PVB, BVB: TDWVersionBlock;
   DLL, EXE: String;
begin
	DLL:= dwGetSpecialFolder(dwspfSystem) + 'Shdocvw.dll';
   with TRegistry.Create(KEY_READ) do begin
      RootKey:= HKEY_LOCAL_MACHINE;
      if OpenKey('\Software\Microsoft\Windows\CurrentVersion\App Paths\IExplore.exe\', FALSE) then begin
		   EXE:= ExtractFilePath(ReadString(''));
      end;
      Free;
   end;
   if FileExists(DLL) then
   	FI:= dwGetFileInfo(DLL)
   else
   	if FileExists(EXE) then FI:= dwGetFileInfo(EXE) else FI:= dwGetFileInfo('IExplore.exe');
   PVB:= dwGetFileProductVersionMS(FI);
   BVB:= dwGetFileProductVersionLS(FI);
   with result do begin
      dwVersionMajor:= PVB.dwVersionMajor;
      dwVersionMinor:= PVB.dwVersionMinor;
      dwBuildMajor:=	  BVB.dwVersionMajor;
      dwBuildMinor:=	  BVB.dwVersionMinor;
   end;
   dwiVersionBlockExSetString(result);
   dwiGetIESuiteName(result);
end;

function dwGetIEVersionLS: TDWVersionBlock;
var
   SL: TStringList;
begin
   SL:= TStringList.Create;
   with SL do begin
      dwStrToList(dwiGetIEVersionString, '.', SL);
      with result do begin
         dwVersionMajor:= StrToIntDef(Strings[2], 0);
         dwVersionMinor:= StrToIntDef(Strings[3], 0);
      end;
      Free;
   end;
end;

function dwGetIEVersionMS: TDWVersionBlock;
var SL: TStringList;
begin
   SL:= TStringList.Create;
   with SL do begin
      dwStrToList(dwiGetIEVersionString, '.', SL);
      with result do begin
         dwVersionMajor:= StrToIntDef(Strings[0], 0);
         dwVersionMinor:= StrToIntDef(Strings[1], 0);
      end;
      Free;
   end;
end;

function dwGetMenuAlignment: TDWMenuAlignment;
begin
   if GetSystemMetrics(SM_MENUDROPALIGNMENT)<>0 then
      result:= dwmaRight else
      result:= dwmaLeft;
end;

function dwGetMinimizedWindowArrangement: TDWMinimizedWindowArrangement;
begin
   case GetSystemMetrics(SM_ARRANGE) of
   ARW_BOTTOMRIGHT: result:= dwmwaBottomRight;
   ARW_HIDE:        result:= dwmwaHide;
   ARW_TOPLEFT:     result:= dwmwaTopLeft;
   ARW_TOPRIGHT:    result:= dwmwaTopRight
   else             result:= dwmwaBottomLeft;
   end;  
end;

function dwGetNTServicePackName: String;
var
   OSV: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSV);
   result:= OSV.szCSDVersion;
end;

function dwGetOS: TDWOSType;
var
   OSV: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSV);
   result:= OS_UNKNOWN;
   if OSV.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS then
      case OSV.dwMinorVersion of
      0:  begin
             case (OSV.dwBuildNumber and $FFFF) of
             950:  begin
                      if (UpperCase(OSV.szCSDVersion) = 'A') then
                         result:= OS_WIN95A
                      else
                         result:= OS_WIN95;
                   end;
             1111: begin
                      result:= OS_WIN95OSR2A;
                   end;
             else  result:= OS_WIN95;
             end;
          end;
      3:  begin
            if (UpperCase(OSV.szCSDVersion) = 'B') then
               result:= OS_WIN95OSR2B
            else
               result:= OS_WIN95OSR2C;
          end;
      10: begin
             case (OSV.dwBuildNumber and $FFFF) of
             1998:   result:= OS_WIN98;
             2222:   result:= OS_WIN98SE;
             else  result:= OS_WIN98;
             end;
          end;
      90: begin
             case (OSV.dwBuildNumber and $FFFF) of
             2476:   result:= OS_WINMEBETA;
             3000:   result:= OS_WINME;
             else    result:= OS_WINME;
             end;
          end;
      end
   else if OSV.dwPlatformId = VER_PLATFORM_WIN32_NT then
      case OSV.dwMajorVersion of
      3: result:= OS_NT351;
      4: case OSV.wProductType of
         VER_NT_DOMAIN_CONTROLLER:result:= OS_NT4_DOMAINCONTROLLER;
         VER_NT_SERVER:           result:= OS_NT4_SERVER;
         VER_NT_WORKSTATION:      result:= OS_NT4_WORKSTATION;
         end;
      5: case OSV.dwMinorVersion of
         0: case OSV.wSuiteMask of
            VER_SUITE_ENTERPRISE: result:= OS_WIN2000_ADVSERVER;
            VER_SUITE_DATACENTER: result:= OS_WIN2000_DATACENTER;
            else                  result:= OS_WIN2000;
            end;
         1: case OSV.wSuiteMask of
            VER_SUITE_PERSONAL:   result:= OS_WINXP_HOME;
            VER_SUITE_ENTERPRISE: result:= OS_DOTNETSERVER;
            else                  result:= OS_WINXP_PROFESSIONAL;
            end;
         2: case OSV.wSuiteMask of
         	VER_SUITE_DATACENTER: result:= OS_WINDOWS_SERVER_2003_DATACENTER;
            VER_SUITE_ENTERPRISE: result:= OS_WINDOWS_SERVER_2003_ENTERPRISE;
            VER_SUITE_SECURITY_APPLIANCE:
            							 result:= OS_WINDOWS_SERVER_2003_SECAPPL;
            VER_SUITE_BLADE:		 result:= OS_WINDOWS_SERVER_2003_WEBEDITION;
            else						 result:= OS_WINDOWS_SERVER_2003;
            end;
         end;
      6: case OSV.dwMinorVersion of
         0:	case (OSV.dwBuildNumber and $FFFF) of
            4051: 					 result:= OS_WINDOWS_LONGHORN_ALPHA1;
         	end;
      	end;
      end
   else begin
      result:= OS_UNKNOWN;
   end;
end;

function dwGetOSName: String;
begin
   case dwGetOS of
   OS_DOTNETSERVER:         					result:= DWLNG_OSName1;
   OS_WIN95:                					result:= DWLNG_OSName2;
   OS_WIN95A:					 					result:= DWLNG_OSName3;
   OS_WIN95OSR2A:				 					result:= DWLNG_OSName4;
   OS_WIN95OSR2B:				 					result:= DWLNG_OSName5;
   OS_WIN95OSR2C:				 					result:= DWLNG_OSName6;
   OS_WIN98:                					result:= DWLNG_OSName7;
   OS_WIN98SE:					 					result:= DWLNG_OSName8;
   OS_WINME:                					result:= DWLNG_OSName9;
   OS_WINMEBETA:            					result:= DWLNG_OSName10;
   OS_NT351:                					result:= DWLNG_OSName11;
   OS_NT4_DOMAINCONTROLLER: 					result:= DWLNG_OSName12;
   OS_NT4_SERVER:           					result:= DWLNG_OSName13;
   OS_NT4_WORKSTATION:      					result:= DWLNG_OSName14;
   OS_WIN2000:              					result:= DWLNG_OSName15;
   OS_WIN2000_ADVSERVER:    					result:= DWLNG_OSName16;
   OS_WIN2000_DATACENTER:   					result:= DWLNG_OSName17;
   OS_WINXP_HOME:           					result:= DWLNG_OSName18;
   OS_WINXP_PROFESSIONAL:   					result:= DWLNG_OSName19;
   OS_WINDOWS_SERVER_2003:  					result:= DWLNG_OSName20;
   OS_WINDOWS_SERVER_2003_DATACENTER:		result:= DWLNG_OSName21;
   OS_WINDOWS_SERVER_2003_ENTERPRISE:		result:= DWLNG_OSName22;
   OS_WINDOWS_SERVER_2003_SECAPPL:			result:= DWLNG_OSName23;
   OS_WINDOWS_SERVER_2003_WEBEDITION:		result:= DWLNG_OSName24;
   OS_UNKNOWN:              					result:= DWLNG_OSName25;
   end;
end;

function dwGetProductType: TOSProductType;
var
   OSV: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSV);
   case OSV.wProductType of
   VER_NT_WORKSTATION:                 result:= OSPT_WORKSTATION;
   VER_NT_DOMAIN_CONTROLLER:           result:= OSPT_DOMAIN_CONTROLLER;
   VER_NT_SERVER:                      result:= OSPT_SERVER;
   else                                result:= OSPT_UNKNOWN;
   end;
end;

function dwGetWindowsLanguageStr: String;
var
  WinLanguage: array [0..50] of char;
begin
  VerLanguageName(GetSystemDefaultLangID, WinLanguage, 50);
  Result := StrPas(WinLanguage);
end;

function dwGetSuiteType: TOSSuiteTypes;
var
   OSV: TOSVersionInfoEx;

   function CmpSuite(const VER_SUITE_x, SuiteMask: DWord; const SuiteType: TOSSuiteType; var SuiteTypes: TOSSuiteTypes): Boolean;
   begin
      result:= ((VER_SUITE_x and SuiteMask) = SuiteMask);
      if result then Include(SuiteTypes, SuiteType);
   end;
begin
   dwGetWindowsVersionEx(OSV);
   with OSV do begin
     CmpSuite(VER_SUITE_BACKOFFICE, wSuiteMask, OSST_BACKOFFICE, result);
     CmpSuite(VER_SUITE_BLADE, wSuiteMask, OSST_DOTNETWEBSERVER, result);
     CmpSuite(VER_SUITE_DATACENTER, wSuiteMask, OSST_DATACENTER, result);
     CmpSuite(VER_SUITE_ENTERPRISE, wSuiteMask, OSST_ENTERPRISE, result);
     CmpSuite(VER_SUITE_PERSONAL, wSuiteMask, OSST_PERSONAL, result);
     CmpSuite(VER_SUITE_SMALLBUSINESS, wSuiteMask, OSST_SMALLBUSINESS, result);
     CmpSuite(VER_SUITE_SMALLBUSINESS_RESTRICTED, wSuiteMask, OSST_SMALLBUSINESS_RESTRICTED, result);
     CmpSuite(VER_SUITE_TERMINAL, wSuiteMask, OSST_TERMINAL, result);
   end;
end;

procedure dwGetWindowsVersionEx(var OSVIE: TOSVersionInfoEx);
var
  lpOS, lpOS2: POsVersionInfoEx;
begin
  GetMem(lpOS, SizeOf(TOsVersionInfo));
  lpOs^.dwOSVersionInfoSize := SizeOf(TOsVersionInfo);
  while dwInternal.getVersionEx(lpOS) = false do
  begin
    GetMem(lpos2, lpos^.dwOSVersionInfoSize +1);
    lpOs2^.dwOSVersionInfoSize := lpOs^.dwOSVersionInfoSize +1;
    FreeMem(lpOs, lpOs^.dwOSVersionInfoSize);
    lpOS := lpOs2;
  end;
  OSVIE.dwMajorVersion := lpOs^.dwMajorVersion;
  OSVIE.dwMinorVersion := lpOs^.dwMinorVersion;
  OSVIE.dwBuildNumber := lpOS^.dwBuildNumber;
  OSVIE.dwPlatformId := lpOS^.dwPlatformId;
  OSVIE.szCSDVersion := lpOS^.szCSDVersion;
  OSVIE.wServicePackMajor := lpOS^.wServicePackMajor;
  OSVIE.wServicePackMinor := lpOS^.wServicePackMinor;
  OSVIE.wProductType := lpOS^.wProductType;
  OSVIE.wReserved := lpOS^.wReserved;
  FreeMem(lpOs, lpOs^.dwOSVersionInfoSize);
end;

function dwIsDomainController: Boolean;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= (OSVersionInfoEx.wProductType = VER_NT_DOMAIN_CONTROLLER);
end;

function dwIsNT351: Boolean;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= ( OSVersionInfoEx.dwPlatformId = VER_PLATFORM_WIN32_NT ) and
            ( OSVersionInfoEx.dwMajorVersion = 3 ) and
            ( OSVersionInfoEx.dwMinorVersion = 51 );
end;

function dwIsNT4: Boolean;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= ( OSVersionInfoEx.dwPlatformId = VER_PLATFORM_WIN32_NT ) and
            ( OSVersionInfoEx.dwMajorVersion = 4 ) and
            ( OSVersionInfoEx.dwMinorVersion = 0 );
end;

function dwIsNTMachine: Boolean;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= ( OSVersionInfoEx.dwPlatformId = VER_PLATFORM_WIN32_NT );
end;

function dwIsNTServer: Boolean;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= (OSVersionInfoEx.wProductType = VER_NT_SERVER);
end;

function dwIsNTWorkstation: Boolean;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= (OSVersionInfoEx.wProductType = VER_NT_WORKSTATION);
end;

function dwIsWin2K: Boolean;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= ( OSVersionInfoEx.dwPlatformId = VER_PLATFORM_WIN32_NT ) and
            ( OSVersionInfoEx.dwMajorVersion = 5 ) and
            ( OSVersionInfoEx.dwMinorVersion = 0 );
end;

function dwIsWin95: Boolean;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= ( OSVersionInfoEx.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS ) and
            ( OSVersionInfoEx.dwMajorVersion = 4 ) and
            ( OSVersionInfoEx.dwMinorVersion = 0 );
end;

function dwIsWin98: Boolean;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= ( OSVersionInfoEx.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS ) and
            ( OSVersionInfoEx.dwMajorVersion = 4 ) and
            ( OSVersionInfoEx.dwMinorVersion = 10 );
end;

function dwIsWinME: Boolean;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= ( OSVersionInfoEx.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS ) and
            ( OSVersionInfoEx.dwMajorVersion = 4 ) and
            ( OSVersionInfoEx.dwMinorVersion = 90 );
end;

function dwIsWinXP: Boolean;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= ( OSVersionInfoEx.dwPlatformId = VER_PLATFORM_WIN32_NT ) and
            ( OSVersionInfoEx.dwMajorVersion = 5 ) and
            ( OSVersionInfoEx.dwMinorVersion = 1 );
end;

function dwLockWorkstation: Boolean;
type
  TLockWorkStation = function: Boolean;
var
  hUser32: HMODULE;
  LockWorkStation: TLockWorkStation;
begin
  result:= FALSE;
  hUser32:= GetModuleHandle('USER32.DLL');
  if hUser32 <> 0 then
  begin
    @LockWorkStation:= GetProcAddress(hUser32, 'LockWorkStation');
    if @LockWorkStation <> nil then
    begin
      LockWorkStation;
      Result := TRUE;
    end;
  end;
end;

procedure dwMinimizeAllWindows;
begin
   keybd_event(VK_LWIN, MapVirtualKey(VK_LWIN, 0), 0, 0);
   keybd_event(Ord('M'), MapVirtualKey(Ord('M'), 0), 0, 0);
   keybd_event(Ord('M'), MapVirtualKey(Ord('M'), 0), KEYEVENTF_KEYUP, 0);
   keybd_event(VK_LWIN, MapVirtualKey(VK_LWIN, 0), KEYEVENTF_KEYUP, 0);
end;

function dwNTServicePackMajorVersion: Integer;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= OSVersionInfoEx.wServicePackMajor;
end;

function dwNTServicePackMinorVersion: Integer;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= OSVersionInfoEx.wServicePackMinor;
end;

function dwSetEnvironmentVariable(const VarName, VarValue: String; const SetAllUsers: Boolean): Boolean;
resourcestring
  REG_MACHINE_LOCATION = 'System\CurrentControlSet\Control\Session Manager\Environment';
  REG_USER_LOCATION = 'Environment';
begin
  with TRegistry.Create do
    try
      if not SetAllUsers then { User Environment Variable }
        Result := OpenKey(REG_USER_LOCATION, True)
      else { System Environment Variable }
      begin
        RootKey := HKEY_LOCAL_MACHINE;
        Result  := OpenKey(REG_MACHINE_LOCATION, True);
      end;
      if Result then
      begin
        WriteString(VarName, VarValue); { Write Registry for Global Environment }
        { Update Current Process Environment Variable }
        SetEnvironmentVariable(PChar(VarName), PChar(VarValue));
        { Send Message To All Top Window for Refresh }
        SendMessage(HWND_BROADCAST, WM_SETTINGCHANGE, 0, Integer(PChar('Environment')));
      end;
    finally
      Free;
    end;
end; { SetGlobalEnvironment }

procedure dwShowDesktop;
begin
   keybd_event(VK_LWIN, MapVirtualKey(VK_LWIN, 0), 0, 0);
   keybd_event(Ord('D'), MapVirtualKey(Ord('D'), 0), 0, 0);
   keybd_event(Ord('D'), MapVirtualKey(Ord('D'), 0), KEYEVENTF_KEYUP, 0);
   keybd_event(VK_LWIN, MapVirtualKey(VK_LWIN, 0), KEYEVENTF_KEYUP, 0);
end;

function dwVirtualMachineIsPresent: Boolean;
asm
  push ebp

  mov  ecx, offset @@exception_handler
  mov  ebp, esp

  push ebx
  push ecx
  push dword ptr fs:[0]
  mov  dword ptr fs:[0], esp

  mov  ebx, 0
  mov  eax, 1

  db 00Fh, 03Fh, 007h, 00Bh

  mov eax, dword ptr ss:[esp]
  mov dword ptr fs:[0], eax
  add esp, 8

  test ebx, ebx
  setz al
  lea esp, dword ptr ss:[ebp-4]
  mov ebx, dword ptr ss:[esp]
  mov ebp, dword ptr ss:[esp+4]
  add esp, 8
  jmp @@ret
  @@exception_handler:
  mov ecx, [esp+0Ch]
  mov dword ptr [ecx+0A4h], -1
  add dword ptr [ecx+0B8h], 4
  xor eax, eax
  ret
  @@ret:
end;

function dwVMWareIsPresent: Boolean;
begin
 Result := False;
{$IFDEF CPU386}
 try
  asm
      mov   eax, 564D5868h
      mov   dx, 5658h
      mov   ecx, 0000000Ah
      xor   ebx, ebx
      in   eax, dx
      cmp   ebx, 564D5868h
      jne   @@exit
      mov   Result, True
  @@exit:
  end;
 except
  Result := False;
 end;
{$ENDIF}
end;

function dwWinBuildNumber: Integer;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= OSVersionInfoEx.dwBuildNumber and $FFFF;
end;

function dwWinMajorVersion: Integer;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= OSVersionInfoEx.dwMajorVersion;
end;

function dwWinMinorVersion: Integer;
var
   OSVersionInfoEx: TOSVersionInfoEx;
begin
   dwGetWindowsVersionEx(OSVersionInfoEx);
   result:= OSVersionInfoEx.dwMinorVersion;
end;

function dwWow64IsRunning(AppHandle: THandle): Boolean;
type
   TIsWow64Process = function(hProcess: THandle;
                              Wow64Process: Boolean):Boolean; StdCall;
const
   DLLName: PChar = 'Kernel32.dll';
   FcnName: PChar = 'IsWow64Process';
var
   DLLHndl: THandle;
   IsWow64,
   bChkWow: Boolean;
   IsWow64Process: TIsWow64Process;
begin
   result:= FALSE;
   IsWow64:= FALSE;
   bChkWow:= FALSE;
   DLLHndl:= LoadLibrary(DLLName);
   if (DLLHndl <> 0) then begin
      IsWow64Process:= GetProcAddress(DLLHndl, FcnName);
      if Assigned(IsWow64Process) then
         bChkWow:= IsWow64Process(AppHandle, IsWow64);
      result:= (bChkWow) and (IsWow64);
   end;
   FreeLibrary(DLLHndl);
end;

function dwIsAdmin: Boolean;
const
  SECURITY_NT_AUTHORITY: TSIDIdentifierAuthority =
    (Value: (0, 0, 0, 0, 0, 5));
  SECURITY_BUILTIN_DOMAIN_RID = $00000020;
  DOMAIN_ALIAS_RID_ADMINS = $00000220;
var
  hAccessToken: THandle;
  ptgGroups: PTokenGroups;
  dwInfoBufferSize: DWORD;
  psidAdministrators: PSID;
  x: Integer;
  bSuccess: BOOL;
begin
  Result   := False;
  bSuccess := OpenThreadToken(GetCurrentThread, TOKEN_QUERY, True,
    hAccessToken);
  if not bSuccess then
  begin
    if GetLastError = ERROR_NO_TOKEN then
      bSuccess := OpenProcessToken(GetCurrentProcess, TOKEN_QUERY,
        hAccessToken);
  end;
  if bSuccess then
  begin
    GetMem(ptgGroups, 1024);
    bSuccess := GetTokenInformation(hAccessToken, TokenGroups,
      ptgGroups, 1024, dwInfoBufferSize);
    CloseHandle(hAccessToken);
    if bSuccess then
    begin
      AllocateAndInitializeSid(SECURITY_NT_AUTHORITY, 2,
        SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS,
        0, 0, 0, 0, 0, 0, psidAdministrators);
      {$R-}
      for x := 0 to ptgGroups.GroupCount - 1 do
        if EqualSid(psidAdministrators, ptgGroups.Groups[x].Sid) then
        begin
          Result := True;
          Break;
        end;
      {$R+}
      FreeSid(psidAdministrators);
    end;
    FreeMem(ptgGroups);
  end;
end;

procedure dwXPUserLogOut;
begin
   keybd_event(VK_LWIN, MapVirtualKey(VK_LWIN, 0), 0, 0);
   keybd_event(Ord('L'), MapVirtualKey(Ord('L'), 0), 0, 0);
   keybd_event(Ord('L'), MapVirtualKey(Ord('L'), 0), KEYEVENTF_KEYUP, 0);
   keybd_event(VK_LWIN, MapVirtualKey(VK_LWIN, 0), KEYEVENTF_KEYUP, 0);
end;

end.
